#!/usr/bin/env python
# -*- coding: utf-8 -*-

""" MultiQC submodule to parse output from GATK varianteval """

import logging
from collections import OrderedDict
from multiqc.plots import bargraph, table

# Initialise the logger
log = logging.getLogger(__name__)

class VariantEvalMixin():

    def parse_gatk_varianteval(self):
        """ Find GATK varianteval logs and parse their data """

        self.gatk_varianteval = dict()
        for f in self.find_log_files('gatk/varianteval', filehandles=True):
            parsed_data = parse_single_report(f['f'])
            if len(parsed_data) > 1:
                if f['s_name'] in self.gatk_varianteval:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
                self.add_data_source(f, section='varianteval')
                self.gatk_varianteval[f['s_name']] = parsed_data

        # Filter to strip out ignored sample names
        self.gatk_varianteval = self.ignore_samples(self.gatk_varianteval)

        n_reports_found = len(self.gatk_varianteval)
        if n_reports_found > 0:
            log.info("Found {} VariantEval reports".format(n_reports_found))

            # Write parsed report data to a file (restructure first)
            self.write_data_file(self.gatk_varianteval, 'multiqc_gatk_varianteval')

            # Get consensus TiTv references
            titv_ref = None
            for s_name in self.gatk_varianteval:
                if titv_ref is None:
                    titv_ref = self.gatk_varianteval[s_name]['titv_reference']
                elif titv_ref != self.gatk_varianteval[s_name]['titv_reference']:
                    titv_ref = 'Multiple'
                    break

            # General Stats Table
            varianteval_headers = dict()
            varianteval_headers['known_titv'] = {
                'title': 'TiTV ratio (known)',
                'description': "TiTV ratio from variants found in '{}'".format(titv_ref),
                'min': 0,
                'scale': 'Blues',
                'shared_key': 'titv_ratio'
            }
            varianteval_headers['novel_titv'] = {
                'title': 'TiTV ratio (novel)',
                'description': "TiTV ratio from variants NOT found in '{}'".format(titv_ref),
                'min': 0,
                'scale': 'Blues',
                'shared_key': 'titv_ratio'
            }
            varianteval_headers['called_titv'] = {
                'title': 'TiTV ratio (called)',
                'description': "TiTV ratio from variants found in '{}'".format(titv_ref),
                'min': 0,
                'scale': 'Blues',
                'shared_key': 'titv_ratio'
            }
            varianteval_headers['filtered_titv'] = {
                'title': 'TiTV ratio (filtered)',
                'description': "TiTV ratio from variants NOT found in '{}'".format(titv_ref),
                'min': 0,
                'scale': 'Blues',
                'shared_key': 'titv_ratio'
            }
            self.general_stats_addcols(self.gatk_varianteval, varianteval_headers, 'GATK VariantEval')

            # Variant Counts plot
            self.add_section (
                name = 'Variant Counts',
                anchor = 'gatk-count-variants',
                plot = count_variants_barplot(self.gatk_varianteval)
            )

            # Compare Overlap Table
            self.add_section (
                name = 'Compare Overlap',
                anchor = 'gatk-compare-overlap',
                plot = comp_overlap_table(self.gatk_varianteval)
            )

        return n_reports_found


def parse_single_report(f):
    """ Parse a gatk varianteval varianteval """
    # Fixme: Separate GATKReport parsing and data subsetting. A GATKReport parser now available from the GATK MultiqcModel.

    data = dict()
    in_CompOverlap = False
    in_CountVariants = False
    in_TiTv = False
    for l in f:
        # Detect section headers
        if '#:GATKTable:CompOverlap' in l:
            in_CompOverlap = True
        elif '#:GATKTable:CountVariants' in l:
            in_CountVariants = True
        elif '#:GATKTable:TiTvVariantEvaluator' in l:
            in_TiTv = True
        else:
            # Parse contents using nested loops
            if in_CompOverlap:
                headers = l.split()
                while in_CompOverlap:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_CompOverlap = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get('Novelty') == 'all' or d.get('Filter') == 'raw':
                        if 'CompFeatureInput' in d:
                            data['reference'] = d['CompFeatureInput']
                        if 'CompRod' in d:
                            data['reference'] = d['CompRod']
                        if 'compRate' in d:
                            data['comp_rate'] = float(d['compRate'])
                        if 'concordantRate' in d:
                            data['concordant_rate'] = float(d['concordantRate'])
                        if 'nEvalVariants' in d:
                            data['eval_variants'] = int(d['nEvalVariants'])
                        if 'novelSites' in d:
                            data['novel_sites'] = int(d['novelSites'])
                    elif d.get('Novelty') == 'known':
                        if 'nEvalVariants' in d:
                            data['known_sites'] = int(d['nEvalVariants'])

            elif in_CountVariants:
                headers = l.split()
                while in_CountVariants:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_CountVariants = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get('Novelty') == 'all' or d.get('Filter') == 'raw':
                        if 'nSNPs' in d:
                            data['snps'] = int(d['nSNPs'])
                        if 'nMNPs' in d:
                            data['mnps'] = int(d['nMNPs'])
                        if 'nInsertions' in d:
                            data['insertions'] = int(d['nInsertions'])
                        if 'nDeletions' in d:
                            data['deletions'] = int(d['nDeletions'])
                        if 'nComplex' in d:
                            data['complex'] = int(d['nComplex'])
                        if 'nSymbolic' in d:
                            data['symbolic'] = int(d['nSymbolic'])
                        if 'nMixed' in d:
                            data['mixed'] = int(d['nMixed'])
                        if 'nNoCalls' in d:
                            data['nocalls'] = int(d['nNoCalls'])
            elif in_TiTv:
                headers = l.split()
                data['titv_reference'] = 'unknown'
                while in_TiTv:
                    l = f.readline().strip("\n")
                    if len(l) < len(headers):
                        in_TiTv = False
                        break
                    d = dict()
                    for i, s in enumerate(l.split()):
                        d[headers[i]] = s
                    if d.get('Novelty') == 'known':
                        if 'CompFeatureInput' in d:
                            data['titv_reference'] = d['CompFeatureInput']
                        if 'CompRod' in d:
                            data['titv_reference'] = d['CompRod']
                        if 'tiTvRatio' in d:
                            data['known_titv'] = float(d['tiTvRatio'])
                    elif d.get('Novelty') == 'novel':
                        if 'tiTvRatio' in d:
                            data['novel_titv'] = float(d['tiTvRatio'])
                    elif d.get('Filter') == 'called':
                        if 'CompFeatureInput' in d:
                            data['titv_reference'] = d['CompFeatureInput']
                        if 'CompRod' in d:
                            data['titv_reference'] = d['CompRod']
                        if 'tiTvRatio' in d:
                            data['called_titv'] = float(d['tiTvRatio'])
                    elif d.get('Filter') == 'filtered':
                        if 'CompFeatureInput' in d:
                            data['titv_reference'] = d['CompFeatureInput']
                        if 'CompRod' in d:
                            data['titv_reference'] = d['CompRod']
                        if 'tiTvRatio' in d:
                            data['filtered_titv'] = float(d['tiTvRatio'])

    return data


def count_variants_barplot(data):
    """ Return HTML for the Variant Counts barplot """
    keys = OrderedDict()
    keys['snps'] = {'name': 'SNPs'}
    keys['mnps'] = {'name': 'MNPs'}
    keys['insertions'] = {'name': 'Insertions'}
    keys['deletions'] = {'name': 'Deletions'}
    keys['complex'] = {'name': 'Complex'}
    keys['symbolic'] = {'name': 'Symbolic'}
    keys['mixed'] = {'name': 'Mixed'}
    keys['nocalls'] = {'name': 'No-calls'}

    plot_conf = {
        'id': 'gatk_varianteval_variant_plot',
        'title': 'GATK VariantEval: Variant Counts',
        'ylab': '# Variants',
        'cpswitch_counts_label': 'Number of Variants'
    }
    return bargraph.plot(data, keys, plot_conf)


def comp_overlap_table(data):
    """Build a table from the comp overlaps output."""
    headers = OrderedDict()
    headers['comp_rate'] = {
        'title': 'Compare rate',
        'description': 'Ratio of known variants found in the reference set.',
        'namespace': 'GATK',
        'min': 0,
        'max': 100,
        'suffix': '%',
        'format': '{:,.2f}',
        'scale': 'Blues',
    }
    headers['concordant_rate'] = {
        'title': 'Concordant rate',
        'description': 'Ratio of variants matching alleles in the reference set.',
        'namespace': 'GATK',
        'min': 0,
        'max': 100,
        'suffix': '%',
        'format': '{:,.2f}',
        'scale': 'Blues',
    }
    headers['eval_variants'] = {
        'title': 'M Evaluated variants',
        'description': 'Number of called variants (millions)',
        'namespace': 'GATK',
        'min': 0,
        'modify': lambda x: float(x) / 1000000.0
    }
    headers['known_sites'] = {
        'title': 'M Known sites',
        'description': 'Number of known variants (millions)',
        'namespace': 'GATK',
        'min': 0,
        'modify': lambda x: float(x) / 1000000.0
    }
    headers['novel_sites'] = {
        'title': 'M Novel sites',
        'description': 'Number of novel variants (millions)',
        'namespace': 'GATK',
        'min': 0,
        'modify': lambda x: float(x) / 1000000.0
    }
    table_html = table.plot(data, headers, {'id': 'gatk_compare_overlap', 'table_title': 'GATK - Compare Overlap'})
    return table_html



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard InsertSizeMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc import config
from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find Picard InsertSizeMetrics reports and parse their data """

    # Set up vars
    self.picard_insertSize_data = dict()
    self.picard_insertSize_histogram = dict()
    self.picard_insertSize_samplestats = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/insertsize', filehandles=True):
        s_name = None
        in_hist = False
        for l in f['f']:

            # Catch the histogram values
            if s_name is not None and in_hist is True:
                try:
                    sections = l.split("\t")
                    ins = int(sections[0])
                    tot_count = sum( [int(x) for x in sections[1:]] )
                    self.picard_insertSize_histogram[s_name][ins] = tot_count
                    self.picard_insertSize_samplestats[s_name]['total_count'] += tot_count
                except ValueError:
                    # Reset in case we have more in this log file
                    s_name = None
                    in_hist = False

            # New log starting
            if 'InsertSizeMetrics' in l and 'INPUT' in l:
                s_name = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])

            if s_name is not None:
                if 'InsertSizeMetrics' in l and '## METRICS CLASS' in l:
                    if s_name in self.picard_insertSize_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                    self.add_data_source(f, s_name, section='InsertSizeMetrics')
                    keys = f['f'].readline().strip("\n").split("\t")
                    vals = f['f'].readline().strip("\n").split("\t")
                    self.picard_insertSize_samplestats[s_name] = {'total_count': 0, 'meansum':0, 'total_pairs':0 }
                    orientation_idx = keys.index('PAIR_ORIENTATION')
                    while len(vals) == len(keys):
                        pair_orientation = vals[orientation_idx]
                        rowkey = '{}_{}'.format(s_name, pair_orientation)
                        self.picard_insertSize_data[rowkey] = OrderedDict()
                        self.picard_insertSize_data[rowkey]['SAMPLE_NAME'] = s_name
                        for i, k in enumerate(keys):
                            try:
                                self.picard_insertSize_data[rowkey][k] = float(vals[i])
                            except ValueError:
                                try:
                                    self.picard_insertSize_data[rowkey][k] = float(vals[i].replace(',','.'))
                                    log.debug("Switching commas for points in '{}': {} - {}".format(f['fn'], vals[i], vals[i].replace(',','.')))
                                except ValueError:
                                    self.picard_insertSize_data[rowkey][k] = vals[i]
                            except IndexError:
                                pass # missing data
                        # Add to mean sums
                        rp = self.picard_insertSize_data[rowkey]['READ_PAIRS']
                        mis = self.picard_insertSize_data[rowkey]['MEAN_INSERT_SIZE']
                        self.picard_insertSize_samplestats[s_name]['meansum'] += (rp * mis)
                        self.picard_insertSize_samplestats[s_name]['total_pairs'] += rp

                        vals = f['f'].readline().strip("\n").split("\t")

                    # Skip lines on to histogram
                    l = f['f'].readline().strip("\n")
                    l = f['f'].readline().strip("\n")

                    self.picard_insertSize_histogram[s_name] = OrderedDict()
                    in_hist = True

        for key in list(self.picard_insertSize_data.keys()):
            if len(self.picard_insertSize_data[key]) == 0:
                self.picard_insertSize_data.pop(key, None)
        for s_name in list(self.picard_insertSize_histogram.keys()):
            if len(self.picard_insertSize_histogram[s_name]) == 0:
                self.picard_insertSize_histogram.pop(s_name, None)
                log.debug("Ignoring '{}' histogram as no data parsed".format(s_name))

    # Calculate summed mean values for all read orientations
    for s_name, v in self.picard_insertSize_samplestats.items():
        self.picard_insertSize_samplestats[s_name]['summed_mean'] = v['meansum'] / v['total_pairs']

    # Calculate summed median values for all read orientations
    for s_name in self.picard_insertSize_histogram:
        j = 0
        for idx, c in self.picard_insertSize_histogram[s_name].items():
            j += c
            if j > (self.picard_insertSize_samplestats[s_name]['total_count'] / 2):
                self.picard_insertSize_samplestats[s_name]['summed_median'] = idx
                break


    # Filter to strip out ignored sample names
    self.picard_insertSize_data = self.ignore_samples(self.picard_insertSize_data)

    if len(self.picard_insertSize_data) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_insertSize_data, 'multiqc_picard_insertSize')

        # Do we have median insert sizes?
        missing_medians = False
        for v in self.picard_insertSize_samplestats.values():
            if 'summed_median' not in v:
                missing_medians = True

        # Add to general stats table
        self.general_stats_headers['summed_median'] = {
            'title': 'Insert Size',
            'description': 'Median Insert Size, all read orientations (bp)',
            'min': 0,
            'suffix': ' bp',
            'format': '{:,.0f}',
            'scale': 'GnBu',
        }
        self.general_stats_headers['summed_mean'] = {
            'title': 'Mean Insert Size',
            'description': 'Mean Insert Size, all read orientations (bp)',
            'min': 0,
            'suffix': ' bp',
            'format': '{:,.0f}',
            'scale': 'GnBu',
            'hidden': False if missing_medians else True
        }
        for s_name in self.picard_insertSize_samplestats:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update( self.picard_insertSize_samplestats[s_name] )

        # Section with histogram plot
        if len(self.picard_insertSize_histogram) > 0:
            # Make a normalised percentage version of the data
            data_percent = {}
            for s_name, data in self.picard_insertSize_histogram.items():
                data_percent[s_name] = OrderedDict()
                total = float( sum( data.values() ) )
                for k, v in data.items():
                    data_percent[s_name][k] = (v/total)*100

            # Allow customisation of how smooth the the plot is
            try:
                insertsize_smooth_points = int(config.picard_config['insertsize_smooth_points'])
                log.debug("Custom Picard insert size smoothing: {}".format(insertsize_smooth_points))
            except (AttributeError, KeyError, ValueError):
                insertsize_smooth_points = 500

            # Plot the data and add section
            pconfig = {
                'smooth_points': insertsize_smooth_points,
                'smooth_points_sumcounts': [True, False],
                'id': 'picard_insert_size',
                'title': 'Picard: Insert Size',
                'ylab': 'Count',
                'xlab': 'Insert Size (bp)',
                'xDecimals': False,
                'tt_label': '<b>{point.x} bp</b>: {point.y:.0f}',
                'ymin': 0,
                'data_labels': [
                    {'name': 'Counts', 'ylab': 'Coverage'},
                    {'name': 'Percentages', 'ylab': 'Percentage of Counts'}
                ]
            }
            self.add_section (
                name = 'Insert Size',
                anchor = 'picard-insertsize',
                description = 'Plot shows the number of reads at a given insert size. Reads with different orientations are summed.',
                plot = linegraph.plot([self.picard_insertSize_histogram, data_percent], pconfig)
            )


    # Return the number of detected samples to the parent module
    return len(self.picard_insertSize_data)



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard AlignmentSummaryMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)

def parse_reports(self):
    """ Find Picard AlignmentSummaryMetrics reports and parse their data """

    # Set up vars
    self.picard_alignment_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/alignment_metrics', filehandles=True):
        parsed_data = dict()
        s_name = None
        keys = None
        for l in f['f']:
            # New log starting
            if 'AlignmentSummaryMetrics' in l and 'INPUT' in l:
                s_name = None
                keys = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])
                    parsed_data[s_name] = dict()

            if s_name is not None:
                if 'AlignmentSummaryMetrics' in l and '## METRICS CLASS' in l:
                    keys = f['f'].readline().strip("\n").split("\t")
                elif keys:
                    vals = l.strip("\n").split("\t")
                    if len(vals) == len(keys):
                        # Ignore the FIRST_OF_PAIR / SECOND_OF_PAIR data to simplify things
                        if vals[0] == 'PAIR' or vals[0] == 'UNPAIRED':
                            for i, k in enumerate(keys):
                                try:
                                    parsed_data[s_name][k] = float(vals[i])
                                except ValueError:
                                    parsed_data[s_name][k] = vals[i]
                    else:
                        s_name = None
                        keys = None

        # Remove empty dictionaries
        for s_name in list(parsed_data.keys()):
            if len(parsed_data[s_name]) == 0:
                parsed_data.pop(s_name, None)

        # Manipulate sample names if multiple baits found
        for s_name in parsed_data.keys():
            if s_name in self.picard_alignment_metrics:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name, section='AlignmentSummaryMetrics')
            self.picard_alignment_metrics[s_name] = parsed_data[s_name]

    # Filter to strip out ignored sample names
    self.picard_alignment_metrics = self.ignore_samples(self.picard_alignment_metrics)

    if len(self.picard_alignment_metrics) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_alignment_metrics, 'multiqc_picard_AlignmentSummaryMetrics')

        # Add to general stats table
        self.general_stats_headers['PCT_PF_READS_ALIGNED'] = {
            'title': '% Aligned',
            'description': 'Percent of aligned reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.0f}',
            'scale': 'RdYlGn',
            'modify': lambda x: self.multiply_hundred(x)
        }
        for s_name in self.picard_alignment_metrics:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update( self.picard_alignment_metrics[s_name] )



        # Make the bar plot of alignment read count
        pdata = dict()
        for s_name in self.picard_alignment_metrics.keys():
            pdata[s_name] = dict()
            # Picard reports both reads for PE data. Divide it by two as most people will expect # clusters
            if self.picard_alignment_metrics[s_name]['CATEGORY'] == 'PAIR':
                pdata[s_name]['total_reads'] = self.picard_alignment_metrics[s_name]['TOTAL_READS'] / 2
                pdata[s_name]['aligned_reads'] = self.picard_alignment_metrics[s_name]['PF_READS_ALIGNED'] / 2
            else:
                pdata[s_name]['total_reads'] = self.picard_alignment_metrics[s_name]['TOTAL_READS']
                pdata[s_name]['aligned_reads'] = self.picard_alignment_metrics[s_name]['PF_READS_ALIGNED']
            pdata[s_name]['unaligned_reads'] = pdata[s_name]['total_reads'] - pdata[s_name]['aligned_reads']

        keys = OrderedDict()
        keys['aligned_reads'] = {'name': 'Aligned Reads'}
        keys['unaligned_reads'] = {'name': 'Unaligned Reads'}

        # Config for the plot
        pconfig = {
            'id': 'picard_aligned_reads',
            'title': 'Picard: Aligned Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
        }

        self.add_section (
            name = 'Alignment Summary',
            anchor = 'picard-alignmentsummary',
            description = "Please note that Picard's read counts are divided by two for paired-end data.",
            plot = bargraph.plot(pdata, keys, pconfig)
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_alignment_metrics)



#!/usr/bin/env python

""" MultiQC module to parse output from HiCUP """

from __future__ import print_function
from collections import OrderedDict
import logging
from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ HiCUP module, parses log files saved by HiCUP. """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='HiCUP', anchor='hicup',
        href='http://www.bioinformatics.babraham.ac.uk/projects/hicup/',
        info="(Hi-C User Pipeline) is a tool for mapping and performing "\
         "quality control on Hi-C data.")

        # Find and load any HiCUP summary reports
        self.hicup_data = dict()
        for f in self.find_log_files('hicup'):
            self.parse_hicup_logs(f)

        # Filter to strip out ignored sample names
        self.hicup_data = self.ignore_samples(self.hicup_data)

        if len(self.hicup_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.hicup_data)))

        # Write parsed data to a file
        self.write_data_file(self.hicup_data, 'multiqc_hicup')

        # Basic Stats Table
        self.hicup_stats_table()

        # Report sections
        self.add_section (
            name = 'Read Truncation',
            anchor = 'hicup-truncating',
            plot = self.hicup_truncating_chart()
        )
        self.add_section (
            name = 'Read Mapping',
            anchor = 'hicup-mapping',
            plot = self.hicup_alignment_chart()
        )

        self.add_section (
            name = 'Read Pair Filtering',
            anchor = 'hicup-filtering',
            plot = self.hicup_filtering_chart()
        )

        # TODO: Is there a log file with this data for a line plot?
        # self.add_section (
        #     name = 'Di-Tag Length Distribution',
        #     anchor = 'hicup-lengths',
        #     plot = self.hicup_lengths_chart()
        # )

        self.add_section (
            name = 'De-Duplication &amp; Di-Tag Separation',
            anchor = 'hicup-deduplication',
            plot = self.hicup_dedup_chart()
        )


    def parse_hicup_logs(self, f):
        """ Parse a HiCUP summary report """
        if not f['fn'].endswith('.txt'):
            return None
        header = []
        lines = f['f'].splitlines()
        for l in lines:
            s = l.split("\t")
            if len(header) == 0:
                if s[0] != 'File':
                    return None
                header = s[1:]
            else:
                s_name = self.clean_s_name(s[0], f['root'])
                if s_name.startswith('HiCUP_output/'):
                    s_name = s_name[13:]
                parsed_data = {}
                for idx, num in enumerate(s[1:]):
                    try:
                        parsed_data[header[idx]] = float(num)
                    except:
                        parsed_data[header[idx]] = num
                parsed_data['Duplicate_Read_Pairs'] = parsed_data['Valid_Pairs'] - parsed_data['Deduplication_Read_Pairs_Uniques']
                if s_name in self.hicup_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name)
                self.hicup_data[s_name] = parsed_data


    def hicup_stats_table(self):
        """ Add core HiCUP stats to the general stats table """
        headers = OrderedDict()
        headers['Percentage_Ditags_Passed_Through_HiCUP'] = {
            'title': '% Passed',
            'description': 'Percentage Di-Tags Passed Through HiCUP',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['Deduplication_Read_Pairs_Uniques'] = {
            'title': '{} Unique'.format(config.read_count_prefix),
            'description': 'Unique Di-Tags ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['Percentage_Uniques'] = {
            'title': '% Duplicates',
            'description': 'Percent Duplicate Di-Tags',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn-rev',
            'modify': lambda x: 100 - x
        }
        headers['Valid_Pairs'] = {
            'title': '{} Valid'.format(config.read_count_prefix),
            'description': 'Valid Pairs ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['Percentage_Valid'] = {
            'title': '% Valid',
            'description': 'Percent Valid Pairs',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['Paired_Read_1'] = {
            'title': '{} Pairs Aligned'.format(config.read_count_prefix),
            'description': 'Paired Alignments ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['Percentage_Mapped'] = {
            'title': '% Aligned',
            'description': 'Percentage of Paired Alignments',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        self.general_stats_addcols(self.hicup_data, headers, 'HiCUP')

    def hicup_truncating_chart (self):
        """ Generate the HiCUP Truncated reads plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Not_Truncated_Reads'] = { 'color': '#2f7ed8', 'name': 'Not Truncated' }
        keys['Truncated_Read']      = { 'color': '#0d233a', 'name': 'Truncated' }

        # Construct a data structure for the plot - duplicate the samples for read 1 and read 2
        data = {}
        for s_name in self.hicup_data:
            data['{} Read 1'.format(s_name)] = {}
            data['{} Read 2'.format(s_name)] = {}
            data['{} Read 1'.format(s_name)]['Not_Truncated_Reads'] = self.hicup_data[s_name]['Not_Truncated_Reads_1']
            data['{} Read 2'.format(s_name)]['Not_Truncated_Reads'] = self.hicup_data[s_name]['Not_Truncated_Reads_2']
            data['{} Read 1'.format(s_name)]['Truncated_Read'] = self.hicup_data[s_name]['Truncated_Read_1']
            data['{} Read 2'.format(s_name)]['Truncated_Read'] = self.hicup_data[s_name]['Truncated_Read_2']

        # Config for the plot
        config = {
            'id': 'hicup_truncated_reads_plot',
            'title': 'HiCUP: Truncated Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(data, keys, config)

    def hicup_alignment_chart (self):
        """ Generate the HiCUP Aligned reads plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Unique_Alignments_Read']   = { 'color': '#2f7ed8', 'name': 'Unique Alignments' }
        keys['Multiple_Alignments_Read'] = { 'color': '#492970', 'name': 'Multiple Alignments' }
        keys['Failed_To_Align_Read']     = { 'color': '#0d233a', 'name': 'Failed To Align' }
        keys['Too_Short_To_Map_Read']    = { 'color': '#f28f43', 'name': 'Too short to map' }

        # Construct a data structure for the plot - duplicate the samples for read 1 and read 2
        data = {}
        for s_name in self.hicup_data:
            data['{} Read 1'.format(s_name)] = {}
            data['{} Read 2'.format(s_name)] = {}
            data['{} Read 1'.format(s_name)]['Unique_Alignments_Read'] = self.hicup_data[s_name]['Unique_Alignments_Read_1']
            data['{} Read 2'.format(s_name)]['Unique_Alignments_Read'] = self.hicup_data[s_name]['Unique_Alignments_Read_2']
            data['{} Read 1'.format(s_name)]['Multiple_Alignments_Read'] = self.hicup_data[s_name]['Multiple_Alignments_Read_1']
            data['{} Read 2'.format(s_name)]['Multiple_Alignments_Read'] = self.hicup_data[s_name]['Multiple_Alignments_Read_2']
            data['{} Read 1'.format(s_name)]['Failed_To_Align_Read'] = self.hicup_data[s_name]['Failed_To_Align_Read_1']
            data['{} Read 2'.format(s_name)]['Failed_To_Align_Read'] = self.hicup_data[s_name]['Failed_To_Align_Read_2']
            data['{} Read 1'.format(s_name)]['Too_Short_To_Map_Read'] = self.hicup_data[s_name]['Too_Short_To_Map_Read_1']
            data['{} Read 2'.format(s_name)]['Too_Short_To_Map_Read'] = self.hicup_data[s_name]['Too_Short_To_Map_Read_2']

        # Config for the plot
        config = {
            'id': 'hicup_mapping_stats_plot',
            'title': 'HiCUP: Mapping Statistics',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(data, keys, config)

    def hicup_filtering_chart(self):
        """ Generate the HiCUP filtering plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Valid_Pairs'] =            { 'color': '#2f7ed8', 'name': 'Valid Pairs' }
        keys['Same_Fragment_Internal'] = { 'color': '#0d233a', 'name': 'Same Fragment - Internal' }
        keys['Same_Circularised'] =      { 'color': '#910000', 'name': 'Same Fragment - Circularised' }
        keys['Same_Dangling_Ends'] =     { 'color': '#8bbc21', 'name': 'Same Fragment - Dangling Ends' }
        keys['Re_Ligation'] =            { 'color': '#1aadce', 'name': 'Re-ligation' }
        keys['Contiguous_Sequence'] =    { 'color': '#f28f43', 'name': 'Contiguous Sequence' }
        keys['Wrong_Size'] =             { 'color': '#492970', 'name': 'Wrong Size' }

        # Config for the plot
        config = {
            'id': 'hicup_filtering_plot',
            'title': 'HiCUP: Filtering Statistics',
            'ylab': '# Read Pairs',
            'cpswitch_counts_label': 'Number of Read Pairs',
            'cpswitch_c_active': False
        }

        return bargraph.plot(self.hicup_data, keys, config)

    def hicup_dedup_chart(self):
        """ Generate the HiCUP Deduplication plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Deduplication_Cis_Close_Uniques'] = { 'color': '#2f7ed8', 'name': 'Unique: cis < 10Kbp' }
        keys['Deduplication_Cis_Far_Uniques']   = { 'color': '#0d233a', 'name': 'Unique: cis > 10Kbp' }
        keys['Deduplication_Trans_Uniques']     = { 'color': '#492970', 'name': 'Unique: trans' }
        keys['Duplicate_Read_Pairs']            = { 'color': '#f28f43', 'name': 'Duplicate read pairs' }

        # Config for the plot
        config = {
            'id': 'hicup_dedup_plot',
            'title': 'HiCUP: De-Duplication Statistics',
            'ylab': '# Di-Tags',
            'cpswitch_counts_label': 'Number of Di-Tags',
            'cpswitch_c_active': False
        }

        return bargraph.plot(self.hicup_data, keys, config)



#!/usr/bin/env python

""" MultiQC module to parse output from featureCounts """

from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='featureCounts',
        anchor='featurecounts', target='Subread featureCounts',
        href='http://bioinf.wehi.edu.au/featureCounts/',
        info="is a highly efficient general-purpose read summarization program"\
        " that counts mapped reads for genomic features such as genes, exons,"\
        " promoter, gene bodies, genomic bins and chromosomal locations.")

        # Find and load any featureCounts reports
        self.featurecounts_data = dict()
        self.featurecounts_keys = list()
        for f in self.find_log_files('featurecounts'):
            self.parse_featurecounts_report(f)

        # Filter to strip out ignored sample names
        self.featurecounts_data = self.ignore_samples(self.featurecounts_data)

        if len(self.featurecounts_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.featurecounts_data)))

        # Write parsed report data to a file
        self.write_data_file(self.featurecounts_data, 'multiqc_featureCounts')

        # Basic Stats Table
        self.featurecounts_stats_table()

        # Assignment bar plot
        self.add_section( plot = self.featureCounts_chart() )


    def parse_featurecounts_report (self, f):
        """ Parse the featureCounts log file. """

        file_names = list()
        parsed_data = dict()
        for l in f['f'].splitlines():
            thisrow = list()
            s = l.split("\t")
            if len(s) < 2:
                continue
            if s[0] == 'Status':
                for f_name in s[1:]:
                    file_names.append(f_name)
            else:
                k = s[0]
                if k not in self.featurecounts_keys:
                    self.featurecounts_keys.append(k)
                for val in s[1:]:
                    try:
                        thisrow.append(int(val))
                    except ValueError:
                        pass
            if len(thisrow) > 0:
                parsed_data[k] = thisrow
        # Check that this actually is a featureCounts file, as format and parsing is quite general
        if 'Assigned' not in parsed_data.keys():
            return None
        for idx, f_name in enumerate(file_names):

            # Clean up sample name
            s_name = self.clean_s_name(f_name, f['root'])

            # Reorganised parsed data for this sample
            # Collect total count number
            data = dict()
            data['Total'] = 0
            for k in parsed_data:
                data[k] = parsed_data[k][idx]
                data['Total'] += parsed_data[k][idx]

            # Calculate the percent aligned if we can
            try:
                data['percent_assigned'] = (float(data['Assigned'])/float(data['Total'])) * 100.0
            except (KeyError, ZeroDivisionError):
                pass

            # Add to the main dictionary
            if len(data) > 1:
                if s_name in self.featurecounts_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, s_name)
                self.featurecounts_data[s_name] = data


    def featurecounts_stats_table(self):
        """ Take the parsed stats from the featureCounts report and add them to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['percent_assigned'] = {
            'title': '% Assigned',
            'description': '% Assigned reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn'
        }
        headers['Assigned'] = {
            'title': '{} Assigned'.format(config.read_count_prefix),
            'description': 'Assigned reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuBu',
            'modify': lambda x: float(x) * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.featurecounts_data, headers)


    def featureCounts_chart (self):
        """ Make the featureCounts assignment rates plot """

        # Config for the plot
        config = {
            'id': 'featureCounts_assignment_plot',
            'title': 'featureCounts: Assignments',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.featurecounts_data, self.featurecounts_keys, config)



from itertools import chain

from multiqc.plots import linegraph

def plot_qahist(samples, file_type, **plot_args):
    """ Create line graph plot of histogram data for BBMap 'qahist' output.

    The 'samples' parameter could be from the bbmap mod_data dictionary:
    samples = bbmap.MultiqcModule.mod_data[file_type]
    """

    sumy = sum([int(samples[sample]['data'][x][0])
                for sample in samples
                for x in samples[sample]['data']])

    cutoff = sumy * 0.999
    all_x = set()
    for item in sorted(chain(*[samples[sample]['data'].items()
                                for sample in samples])):
        # Skip plotting values for this x if missing columns.
        if len(item[1]) < 6:
            continue
        all_x.add(item[0])


    columns_to_plot = {
        'Match': {
            0: 'Match',
        },
        'Sub': {
            1: 'Sub',
        },
        'Ins': {
            2: 'Ins',
        },
        'Del': {
            3: 'Del',
        },
        'TrueQuality': {
            4: 'TrueQuality',
        },
        'TrueQualitySub': {
            5: 'TrueQualitySub',
        },
    }

    plot_data = []
    for column_type in columns_to_plot:
        plot_data.append(
            {
                sample+'.'+column_name: {
                    x: samples[sample]['data'][x][column] if x in samples[sample]['data'] else 0
                    for x in all_x
                }
                for sample in samples
                for column, column_name in columns_to_plot[column_type].items()
            }
        )

    plot_params = {
            'id': 'bbmap-' + file_type + '_plot',
            'title': 'BBTools: ' + plot_args['plot_title'],
            'xlab': 'Quality score',
            'ylab': 'Match count',
            'data_labels': [
                {'name': 'Match', 'ylab': 'Match count'},
                {'name': 'Substitution', 'ylab': 'Substitution count'},
                {'name': 'Insertion', 'ylab': 'Insertion count'},
                {'name': 'Deletion', 'ylab': 'Deletion count'},
                {'name': 'TrueQuality', 'ylab': 'Count'},
                {'name': 'TrueQualitySubtitution', 'ylab': 'Count'},
            ]
    }

    plot_params.update(plot_args['plot_params'])
    plot = linegraph.plot(
        plot_data,
        plot_params
    )

    return plot



#!/usr/bin/env python

""" MultiQC module to parse relatedness output from vcftools relatedness """

import csv
import logging
from collections import defaultdict
from multiqc.plots import heatmap

# Initialise the logger
log = logging.getLogger(__name__)


class Relatedness2Mixin():
    def parse_relatedness2(self):
        matrices = {}
        for f in self.find_log_files('vcftools/relatedness2', filehandles=True):
            m = _Relatedness2Matrix(f)
            if m.data and m.x_labels and m.y_labels:
                matrices[f['s_name']] = m

        matrices = self.ignore_samples(matrices)

        if len(matrices) == 0:
            return 0

        log.info('Found {} valid relatedness2 matrices'.format(len(matrices)))

        helptext = '''
        `RELATEDNESS_PHI` gives a relatedness score between two samples. A higher score indicates a higher degree of
        relatedness, up to a maximum of 0.5. Samples are sorted alphabetically on each axis, and specific IDs can be
        found in the graph with the Highlight tab.
        '''

        idx = 0
        for name, m in matrices.items():
            idx += 1
            self.add_section(
                name = 'Relatedness2',
                anchor = 'vcftools-relatedness2-{}'.format(idx),
                description = "**Input:** `{}`.\n\n Heatmap of `RELATEDNESS_PHI` values from the output of vcftools relatedness2.".format(name),
                helptext = helptext,
                plot = heatmap.plot(
                    m.data,
                    xcats = m.x_labels,
                    ycats = m.y_labels,
                    pconfig = {
                        'id': 'vcftools-relatedness2-heatmap-{}'.format(idx),
                        'title': 'VCFTools: Relatedness2',
                        'square': True,
                        'decimalPlaces': 7
                    }
                )
            )

        return len(matrices)


class _Relatedness2Matrix():
    def __init__(self, relatedness_file):
        self.data = []
        self.x_labels = set()
        self.y_labels = set()

        self.parse(relatedness_file['f'])

    def parse(self, f):
        rels = defaultdict(dict)
        r = csv.DictReader(f, delimiter='\t')
        for line in r:
            self.x_labels.add(line['INDV1'])
            self.y_labels.add(line['INDV2'])

            rels[line['INDV1']][line['INDV2']] = float(line['RELATEDNESS_PHI'])

        # impose alphabetical order and avoid json serialisation errors in utils.report
        self.x_labels = sorted(self.x_labels)
        self.y_labels = sorted(self.y_labels)

        for x in self.x_labels:
            line = []
            for y in self.y_labels:
                line.append(rels[x][y])
            self.data.append(line)



#!/usr/bin/env python

""" MultiQC module to parse output from Trimmomatic """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='Trimmomatic',
            anchor='trimmomatic',
            href='http://www.usadellab.org/cms/?page=trimmomatic',
            info="is a flexible read trimming tool for Illumina NGS data."
        )

        # Parse logs
        self.trimmomatic = dict()
        for f in self.find_log_files('trimmomatic', filehandles=True):
            self.parse_trimmomatic(f)

        # Filter to strip out ignored sample names
        self.trimmomatic = self.ignore_samples(self.trimmomatic)

        if len(self.trimmomatic) == 0:
            raise UserWarning

        log.info("Found {} logs".format(len(self.trimmomatic)))
        self.write_data_file(self.trimmomatic, 'multiqc_trimmomatic')

        # Add drop rate to the general stats table
        headers = OrderedDict()
        headers['dropped_pct'] = {
            'title': '% Dropped',
            'description': '% Dropped reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'OrRd'
        }
        self.general_stats_addcols(self.trimmomatic, headers)

        # Make barplot
        self.trimmomatic_barplot()

    def parse_trimmomatic(self, f):
        s_name = None
        if getattr(config, 'trimmomatic', {}).get('s_name_filenames', False):
            s_name = f['s_name']
        for l in f['f']:
            # Get the sample name
            if s_name is None and 'Trimmomatic' in l and 'Started with arguments:' in l:
                # Match everything up until the first .fastq or .fq
                match = re.search('Trimmomatic[SP]E: Started with arguments:.+?(?=\.fastq|\.fq)', l)
                if match:
                    # backtrack from the end to the first space
                    s_name = match.group().split()[-1]
                    s_name = self.clean_s_name(s_name, f['root'])
                    if s_name in self.trimmomatic:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                else:
                    # Try looking on the next line instead, sometimes have a line break (see issue #212)
                    l = next(f['f'])
                    match = re.search('.+?(?=\.fastq|\.fq)', l)
                    if match:
                        # backtrack from the end to the first space
                        s_name = match.group().split()[-1]
                        s_name = self.clean_s_name(s_name, f['root'])
                        if s_name in self.trimmomatic:
                            log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))

            # Get single end stats
            if 'Input Reads' in l and s_name is not None:
                match = re.search('Input Reads: (\d+) Surviving: (\d+) \(([\d\.,]+)%\) Dropped: (\d+) \(([\d\.,]+)%\)', l)
                if match:
                    self.trimmomatic[s_name] = dict()
                    self.trimmomatic[s_name]['input_reads'] = float( match.group(1) )
                    self.trimmomatic[s_name]['surviving'] = float( match.group(2) )
                    self.trimmomatic[s_name]['surviving_pct'] = float( match.group(3).replace(',','.') )
                    self.trimmomatic[s_name]['dropped'] = float( match.group(4) )
                    self.trimmomatic[s_name]['dropped_pct'] = float( match.group(5).replace(',','.') )
                    s_name = None

            # Get paired end stats
            if 'Input Read Pairs' in l and s_name is not None:
                match = re.search('Input Read Pairs: (\d+) Both Surviving: (\d+) \(([\d\.,]+)%\) Forward Only Surviving: (\d+) \(([\d\.,]+)%\) Reverse Only Surviving: (\d+) \(([\d\.,]+)%\) Dropped: (\d+) \(([\d\.,]+)%\)', l)
                if match:
                    self.trimmomatic[s_name] = dict()
                    self.trimmomatic[s_name]['input_read_pairs'] = float( match.group(1) )
                    self.trimmomatic[s_name]['surviving'] = float( match.group(2) )
                    self.trimmomatic[s_name]['surviving_pct'] = float( match.group(3).replace(',','.') )
                    self.trimmomatic[s_name]['forward_only_surviving'] = float( match.group(4) )
                    self.trimmomatic[s_name]['forward_only_surviving_pct'] = float( match.group(5).replace(',','.') )
                    self.trimmomatic[s_name]['reverse_only_surviving'] = float( match.group(6) )
                    self.trimmomatic[s_name]['reverse_only_surviving_pct'] = float( match.group(7).replace(',','.') )
                    self.trimmomatic[s_name]['dropped'] = float( match.group(8) )
                    self.trimmomatic[s_name]['dropped_pct'] = float( match.group(9).replace(',','.') )
                    s_name = None


    def trimmomatic_barplot (self):
        """ Make the HighCharts HTML to plot the trimmomatic rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['surviving'] =              { 'color': '#437bb1', 'name': 'Surviving Reads' }
        keys['both_surviving'] =         { 'color': '#f7a35c', 'name': 'Both Surviving' }
        keys['forward_only_surviving'] = { 'color': '#e63491', 'name': 'Forward Only Surviving' }
        keys['reverse_only_surviving'] = { 'color': '#b1084c', 'name': 'Reverse Only Surviving' }
        keys['dropped'] =                { 'color': '#7f0000', 'name': 'Dropped' }

        # Config for the plot
        pconfig = {
            'id': 'trimmomatic_plot',
            'title': 'Trimmomatic: Surviving Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        self.add_section( plot = bargraph.plot(self.trimmomatic, keys, pconfig) )



#!/usr/bin/env python

""" MultiQC module to parse output from RSeQC """

from collections import OrderedDict
import logging
import os

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ RSeQC is a collection of scripts. This MultiQC module
    supports some but not all. The code for each script is split
    into its own file and adds a section to the module ooutput if
    logs are found."""

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='RSeQC', anchor='rseqc',
        href="http://rseqc.sourceforge.net/",
        info="package provides a number of useful modules that can"\
        " comprehensively evaluate high throughput RNA-seq data.")

        # Set up class objects to hold parsed data
        self.general_stats_headers = OrderedDict()
        self.general_stats_data = dict()
        n = dict()

        # Get the list of submodules (can be customised)
        rseqc_sections = getattr(config, 'rseqc_sections', [])
        if len(rseqc_sections) == 0:
            rseqc_sections = [
                'read_distribution',
                'gene_body_coverage',
                'inner_distance',
                'read_gc',
                'read_duplication',
                'junction_annotation',
                'junction_saturation',
                'infer_experiment',
                'bam_stat'
            ]

        # Add self.js to be included in template
        self.js = { 'assets/js/multiqc_rseqc.js' : os.path.join(os.path.dirname(__file__), 'assets', 'js', 'multiqc_rseqc.js') }

        # Call submodule functions
        for sm in rseqc_sections:
            try:
                # Import the submodule and call parse_reports()
                #   Function returns number of parsed logs
                module = __import__('multiqc.modules.rseqc.{}'.format(sm), fromlist=[''])
                n[sm] = getattr(module, 'parse_reports')(self)
                if n[sm] > 0:
                    log.info("Found {} {} reports".format(n[sm], sm))
            except (ImportError, AttributeError):
                log.warning("Could not find RSeQC Section '{}'".format(sm))

        # Exit if we didn't find anything
        if sum(n.values()) == 0:
            raise UserWarning

        # Add to the General Stats table (has to be called once per MultiQC module)
        self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)





#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard RrbsSummaryMetrics """

from collections import OrderedDict
import logging
import os
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)

def parse_reports(self):
    """ Find Picard RrbsSummaryMetrics reports and parse their data """

    # Set up vars
    self.picard_rrbs_metrics = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/rrbs_metrics', filehandles=True):
        parsed_data = dict()
        s_name = None
        keys = None
        for l in f['f']:
            # New log starting
            if 'CollectRrbsMetrics' in l and 'INPUT' in l:
                s_name = None
                keys = None
                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])
                    parsed_data[s_name] = dict()

            if s_name is not None:
                if 'RrbsSummaryMetrics' in l and '## METRICS CLASS' in l:
                    keys = f['f'].readline().strip("\n").split("\t")
                elif keys:
                    vals = l.strip("\n").split("\t")
                    if len(vals) == len(keys):
                        for i, k in enumerate(keys):
                            try:
                                parsed_data[s_name][k] = float(vals[i])
                            except ValueError:
                                parsed_data[s_name][k] = vals[i]
                    else:
                        s_name = None
                        keys = None

        # Remove empty dictionaries
        for s_name in list(parsed_data.keys()):
            if len(parsed_data[s_name]) == 0:
                parsed_data.pop(s_name, None)

        # Collect parsed data
        for s_name in parsed_data.keys():
            if s_name in self.picard_rrbs_metrics:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name, section='RrbsSummaryMetrics')
            self.picard_rrbs_metrics[s_name] = parsed_data[s_name]

    # Filter to strip out ignored sample names
    self.picard_rrbs_metrics = self.ignore_samples(self.picard_rrbs_metrics)

    if len(self.picard_rrbs_metrics) > 0:

        # Write parsed data to a file
        self.write_data_file(self.picard_rrbs_metrics, 'multiqc_picard_RrbsSummaryMetrics')

        # Add to general stats table
        self.general_stats_headers['PCT_CPG_BASES_CONVERTED'] = {
            'title': '% CpG Methylated',
            'description': 'Percentage of times a CpG cytosine was converted',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.0f}',
            'scale': 'RdYlGn-rev',
            'modify': lambda x: 100 - self.multiply_hundred(x)
        }
        self.general_stats_headers['PCT_NON_CPG_BASES_CONVERTED'] = {
            'title': '% Non-CpG Methylated',
            'description': 'Percentage of times a non-CpG cytosine was converted',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.0f}',
            'scale': 'RdYlGn',
            'modify': lambda x: 100 - self.multiply_hundred(x)
        }
        self.general_stats_headers['MEDIAN_CPG_COVERAGE'] = {
            'title': 'Median CpG Cov',
            'description': 'Median coverage of CpG sites',
            'min': 0
        }
        for s_name in self.picard_rrbs_metrics:
            if s_name not in self.general_stats_data:
                self.general_stats_data[s_name] = dict()
            self.general_stats_data[s_name].update( self.picard_rrbs_metrics[s_name] )



        # Make the bar plot of converted bases
        pdata_cpg = dict()
        pdata_noncpg = dict()
        for s_name in self.picard_rrbs_metrics.keys():
            pdata_cpg[s_name] = dict()
            pdata_cpg[s_name]['converted'] = self.picard_rrbs_metrics[s_name]['CPG_BASES_CONVERTED']
            pdata_cpg[s_name]['not_converted'] = self.picard_rrbs_metrics[s_name]['CPG_BASES_SEEN'] - self.picard_rrbs_metrics[s_name]['CPG_BASES_CONVERTED']
            pdata_noncpg[s_name] = dict()
            pdata_noncpg[s_name]['converted'] = self.picard_rrbs_metrics[s_name]['NON_CPG_BASES']
            pdata_noncpg[s_name]['not_converted'] = self.picard_rrbs_metrics[s_name]['NON_CPG_BASES'] - self.picard_rrbs_metrics[s_name]['NON_CPG_CONVERTED_BASES']

        keys = OrderedDict()
        keys['not_converted'] = {'name': 'Unconverted Bases (Methylated)'}
        keys['converted'] = {'name': 'Converted Bases (Unmethylated)'}

        # Config for the plot
        pconfig = {
            'id': 'picard_rrbs_converted_bases_plot',
            'title': 'Picard: Converted Bases',
            'ylab': '# CpG Bases',
            'cpswitch_counts_label': 'Number of Bases',
            'data_labels': [
                {'name': 'CpG', 'ylab': '# CpG Bases'},
                {'name': 'Non-CpG', 'ylab': '# Non-CpG Bases'}
            ]
        }

        self.add_section (
            name = 'RRBS Converted Bases',
            anchor = 'picard-rrbssummary-convertedbases',
            plot = bargraph.plot([pdata_cpg, pdata_noncpg], [keys, keys], pconfig)
        )


        # Make the bar plot of processed reads
        pdata = dict()
        for s_name in self.picard_rrbs_metrics.keys():
            pdata[s_name] = dict()
            pdata[s_name]['with_no_cpg'] = self.picard_rrbs_metrics[s_name]['READS_WITH_NO_CPG']
            pdata[s_name]['ignored_short'] = self.picard_rrbs_metrics[s_name]['READS_IGNORED_SHORT']
            pdata[s_name]['ignored_mismatches'] = self.picard_rrbs_metrics[s_name]['READS_IGNORED_MISMATCHES']
            pdata[s_name]['not_ignored'] = (
                self.picard_rrbs_metrics[s_name]['READS_ALIGNED'] -
                pdata[s_name]['with_no_cpg'] -
                pdata[s_name]['ignored_short'] -
                pdata[s_name]['ignored_mismatches']
            )

        keys = OrderedDict()
        keys['not_ignored'] = {'name': 'Utilised reads'}
        keys['with_no_cpg'] = {'name': 'Ignored (no CpG sites)'}
        keys['ignored_short'] = {'name': 'Ignored (too short)'}
        keys['ignored_mismatches'] = {'name': 'Ignored (exceeded mismatch threshold)'}

        # Config for the plot
        pconfig = {
            'id': 'picard_rrbs_ignored_reads_plot',
            'title': 'Picard: RRBS Read Counts',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        self.add_section (
            name = 'RRBS Read Counts',
            anchor = 'picard-rrbssummary-readcounts',
            plot = bargraph.plot(pdata, keys, pconfig)
        )

    # Return the number of detected samples to the parent module
    return len(self.picard_rrbs_metrics)



#!/usr/bin/env python

""" MultiQC submodule to parse output from RSeQC infer_experiment.py
http://rseqc.sourceforge.net/#infer-experiment-py """

from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find RSeQC infer_experiment reports and parse their data """

    # Set up vars
    self.infer_exp = dict()
    regexes = {
        'pe_sense': r"\"1\+\+,1--,2\+-,2-\+\": (\d\.\d+)",
        'pe_antisense': r"\"1\+-,1-\+,2\+\+,2--\": (\d\.\d+)",
        'se_sense': r"\"\+\+,--\": (\d\.\d+)",
        'se_antisense': r"\+-,-\+\": (\d\.\d+)",
        'failed': r"Fraction of reads failed to determine: (\d\.\d+)"
    }

    # Go through files and parse data using regexes
    for f in self.find_log_files('rseqc/infer_experiment'):
        d = dict()
        for k, r in regexes.items():
            r_search = re.search(r, f['f'], re.MULTILINE)
            if r_search:
                d[k] = float(r_search.group(1))

        if len(d) > 0:
            if f['s_name'] in self.infer_exp:
                log.debug("Duplicate sample name found! Overwriting: {}".format(f['s_name']))
            self.add_data_source(f, section='infer_experiment')
            self.infer_exp[f['s_name']] = d

    # Filter to strip out ignored sample names
    self.infer_exp = self.ignore_samples(self.infer_exp)

    if len(self.infer_exp) > 0:

        # Write to file
        self.write_data_file(self.infer_exp, 'multiqc_rseqc_infer_experiment')

        # Merge PE and SE for plot
        pdata = dict()
        for s_name, vals in self.infer_exp.items():
            pdata[s_name] = dict()
            for k, v in vals.items():
                v *= 100.0 # Multiply to get percentage
                if k[:2] == 'pe' or k[:2] == 'se':
                    k = k[3:]
                pdata[s_name][k] = v + pdata[s_name].get(k, 0)

        # Plot bar graph of groups
        keys = OrderedDict()
        keys['sense'] = {'name': "Sense"}
        keys['antisense'] = {'name': "Antisense"}
        keys['failed'] = {'name': "Undetermined"}

        # Config for the plot
        pconfig = {
            'id': 'rseqc_infer_experiment_plot',
            'title': 'RSeQC: Infer experiment',
            'ylab': '% Tags',
            'ymin': 0,
            'ymax': 100,
            'tt_percentages': False,
            'ylab_format': '{value}%',
            'cpswitch': False
        }

        self.add_section (
            name = 'Infer experiment',
            anchor = 'rseqc-infer_experiment',
            description = '<a href="http://rseqc.sourceforge.net/#infer-experiment-py" target="_blank">Infer experiment</a>' \
                " counts the percentage of reads and read pairs that match the strandedness of overlapping transcripts." \
                " It can be used to infer whether RNA-seq library preps are stranded (sense or antisense).",
            plot = bargraph.plot(pdata, keys, pconfig)
        )

    # Return number of samples found
    return len(self.infer_exp)




#!/usr/bin/env python

""" MultiQC submodule to parse output from deepTools estimateReadFiltering """

import logging
from collections import OrderedDict

from multiqc.plots import table

# Initialise the logger
log = logging.getLogger(__name__)

class estimateReadFilteringMixin():
    def parse_estimateReadFiltering(self):
        """Find estimateReadFiltering output. Only the output from --table is supported."""
        self.deeptools_estimateReadFiltering = dict()
        for f in self.find_log_files('deeptools/estimateReadFiltering'):
            parsed_data = self.parseEstimateReadFilteringFile(f)
            for k, v in parsed_data.items():
                if k in self.deeptools_estimateReadFiltering:
                    log.warning("Replacing duplicate sample {}.".format(k))
                self.deeptools_estimateReadFiltering[k] = v

            if len(parsed_data) > 0:
                self.add_data_source(f, section='estimateReadFiltering')

        self.deeptools_estimateReadFiltering = self.ignore_samples(self.deeptools_estimateReadFiltering)

        if len(self.deeptools_estimateReadFiltering) > 0:
            header = OrderedDict()
            header["M Entries"] = {
                'title': 'M entries',
                'description': 'Number of entries in the file (millions)'}
            header["pct_Aligned"] = {
                'title': '% Aligned',
                'description': 'Percent of aligned entries',
                'scale': 'YlGn',
                'min': 0,
                'max': 100
            }
            header["pct_Filtered"] = {
                'title': '% Tot. Filtered',
                'description': 'Percent of alignment that would be filtered for any reason.',
                'scale': 'OrRd',
                'min': 0,
                'max': 100
            }
            header["pct_Blacklisted"] = {
                'title': '% Blacklisted',
                'description': 'Percent of alignments falling (at least partially) inside a blacklisted region',
                'scale': 'YlOrRd',
                'min': 0,
                'max': 100
            }
            header["pct_MAPQ"] = {
                'title': '% MAPQ', 'description':
                'Percent of alignments having MAPQ scores below the specified threshold',
                'scale': 'YlOrBn',
                'min': 0,
                'max': 100
            }
            header["pct_Missing_Flags"] = {
                'title': '% Missing Flags',
                'description': 'Percent of alignments lacking at least on flag specified by --samFlagInclude',
                'scale': 'PuRd',
                'min': 0,
                'max': 100
            }
            header["pct_Forbidden_Flags"] = {
                'title': '% Forbidden Flags',
                'description': 'Percent of alignments having at least one flag specified by --samFlagExclude',
                'scale': 'OrRd',
                'min': 0,
                'max': 100
            }
            header["pct_deepTools_Dupes"] = {
                'title': '% deepTools Dupes',
                'description': 'Percent of alignments marked by deepTools as being duplicates',
                'scale': 'PuRd',
                'min': 0,
                'max': 100
            }
            header["pct_Duplication"] = {
                'title': '% Duplication',
                'description': 'Percent of alignments originally marked as being duplicates',
                'scale': 'OrRd',
                'min': 0,
                'max': 100
            }
            header["pct_Singletons"] = {
                'title': '% Singletons',
                 'description': 'Percent of alignments that are singletons (i.e., paired-end reads where the mates don\'t align as a pair',
                 'scale': 'OrRd',
                 'min': 0,
                 'max': 100
                }
            header["pct_Strand_Filtered"] = {
                'title': '% Strand Filtered',
                'description': 'Percent of alignments arising from the wrong strand',
                'scale': 'OrRd',
                'min': 0,
                'max': 100
            }

            tdata = dict()
            for k, v in self.deeptools_estimateReadFiltering.items():
                tdata[k] = {
                    'M Entries': v['total'] / 1000000.0,
                    'pct_Aligned': 100. * v['mapped'] / float(v['total']),
                    'pct_Filtered': 100. * v['filtered'] / float(v['total']),
                    'pct_Blacklisted': 100. * v['blacklisted'] / float(v['total']),
                    'pct_Below_MAPQ': 100. * v['mapq'] / float(v['total']),
                    'pct_Missing_Flags': 100. * v['required flags'] / float(v['total']),
                    'pct_Forbidden_Flags': 100. * v['excluded flags'] / float(v['total']),
                    'pct_deepTools_Dupes': 100. * v['internal dupes'] / float(v['total']),
                    'pct_Duplication': 100. * v['dupes'] / float(v['total']),
                    'pct_Singletons': 100. * v['singletons'] / float(v['total']),
                    'pct_Strand_Filtered': 100. * v['strand'] / float(v['total'])
                }

            config = {'namespace': 'deepTools bamPEFragmentSize'}
            self.add_section(
                name = "Filtering metrics",
                anchor = "estimateReadFiltering",
                description = "Estimated percentages of alignments filtered independently for each setting in `estimateReadFiltering`",
                plot = table.plot(tdata, header, config)
            )

        return len(self.deeptools_estimateReadFiltering)

    def parseEstimateReadFilteringFile(self, f):
        d = {}
        firstLine = True
        for line in f['f'].splitlines():
            if firstLine:
                firstLine = False
                continue
            cols = line.strip().split("\t")

            if len(cols) != 12:
                # This is not really the output from estimateReadFiltering!
                log.warning("{} was initially flagged as the tabular output from estimateReadFiltering, but that seems to not be the case. Skipping...".format(f['fn']))
                return dict()

            s_name = self.clean_s_name(cols[0], f['root'])
            if s_name in d:
                log.debug("Replacing duplicate sample {}.".format(s_name))
            d[s_name] = dict()

            try:
                d[s_name]["total"] = self._int(cols[1])
                d[s_name]["mapped"] = self._int(cols[2])
                d[s_name]["blacklisted"] = self._int(cols[3])
                d[s_name]["filtered"] = float(cols[4])
                d[s_name]["mapq"] = float(cols[5])
                d[s_name]["required flags"] = float(cols[6])
                d[s_name]["excluded flags"] = float(cols[7])
                d[s_name]["internal dupes"] = float(cols[8])
                d[s_name]["dupes"] = float(cols[9])
                d[s_name]["singletons"] = float(cols[10])
                d[s_name]["strand"] = float(cols[11])
            except:
                # Obviously this isn't really the output from estimateReadFiltering
                log.warning("{} was initially flagged as the output from estimateReadFiltering, but that seems to not be the case. Skipping...".format(f['fn']))
                return dict()
        return d



# !/usr/bin/env python

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name='SeqyClean',
            anchor='seqyclean',
            href='https://github.com/ibest/seqyclean',
            info='is a pre-processing tool for NGS data that filters adapters, vectors, and contaminants while quality trimming.'
        )

        # Parse logs
        self.seqyclean_data = dict()
        for f in self.find_log_files('seqyclean'):
            rows = f['f'].splitlines()
            headers = rows[0].split("\t")
            cols = rows[1].split("\t")

            self.seqyclean_data[f['s_name']] = dict()
            for header, col in zip(headers, cols):
                # Attempt to convert into a float if we can
                try:
                    col = float(col)
                except (ValueError, TypeError):
                    pass
                self.seqyclean_data[f['s_name']].update({ header : col })

        if len(self.seqyclean_data) == 0:
            raise UserWarning

        self.seqyclean_data = self.ignore_samples(self.seqyclean_data)

        if len(self.seqyclean_data) == 0:
            raise UserWarning

        log.info("Found {} logs".format(len(self.seqyclean_data)))

        # Adding the bar plots
        self.add_section(
            name = 'Summary',
            anchor = 'seqyclean-summary',
            description = 'This plot shows the number of reads that were kept and discarded.',
            plot = self.seqyclean_summary()
        )

        self.add_section(
            name = 'Annotations',
            anchor = 'seqyclean-annotation',
            description = 'This plot shows how reads were annotated.',
            plot = self.seqyclean_analysis()
        )

        self.add_section(
            name = 'Discarded',
            anchor = 'seqyclean-discarded',
            description = 'This plot shows the breakdown of reasons for why reads were discarded.',
            plot = self.seqyclean_discarded()
        )

        # Write the results to a file
        self.write_data_file(self.seqyclean_data, 'multiqc_seqyclean')

        # Adding to the general statistics table
        self.seqyclean_general_stats_table()

    def seqyclean_summary(self):
        config = {
            'id': 'seqyclean-summary-plot',
            'title': 'SeqyClean: Summary',
            'ylab': 'Number of Reads'
        }
        keys =[
            'PairsKept', # for paired end
            'PairsDiscarded',
            'PE1DiscardedTotal',
            'PE2DiscardedTotal',
            'SEReadsKept', # single end
            'SEDiscardedTotal',
            'ReadsKept', # 454
            'DiscardedTotal'
        ]
        return bargraph.plot(self.seqyclean_data, self._clean_keys(keys), config)

    def seqyclean_analysis(self):
        config = {
            'id': 'seqyclean-read-annotation-plot',
            'title': 'SeqyClean: Read Annotations',
            'ylab': 'Number of Reads'
        }
        keys = [
            'PE1TruSeqAdap_found', # for paired end
            'PE1ReadsWVector_found',
            'PE1ReadsWContam_found',
            'PE2TruSeqAdap_found',
            'PE2ReadsWVector_found',
            'PE2ReadsWContam_found',
            'SETruSeqAdap_found', # for single end
            'SEReadsWVector_found',
            'SEReadsWContam_found',
            'left_mid_tags_found', # 454 data
            'right_mid_tags_found',
            'ReadsWithVector_found',
            'ReadsWithContam_found'
        ]
        return bargraph.plot(self.seqyclean_data, self._clean_keys(keys), config)

    def seqyclean_discarded(self):
        config = {
            'id': 'seqyclean-discarded-reads-plot',
            'title': 'SeqyClean: Discarded Reads',
            'ylab': 'Number of Reads'
        }
        keys = [
            'SEDiscByContam', # single end
            'SEDiscByLength',
            'PE1DiscByContam', # paired end
            'PE1DiscByLength',
            'PE2DiscByContam',
            'PE2DiscByLength',
            'DiscByContam', # 454 data
            'DiscByLength'
        ]
        return bargraph.plot(self.seqyclean_data, self._clean_keys(keys), config)

    def seqyclean_general_stats_table(self):
        headers = OrderedDict()
        # Paired and single end
        headers['Perc_Kept'] = {
            'title': '% Kept',
            'description': 'The percentage of reads remaining after cleaning',
            'scale': 'YlGn',
            'suffix': '%',
            'max': 100,
            'min': 0
        }
        # 454
        headers['PercentageKept'] = {
            'title': '% Kept',
            'description': 'The percentage of reads remaining after cleaning',
            'scale': 'YlGn',
            'suffix': '%',
            'max': 100,
            'min': 0
        }
        self.general_stats_addcols(self.seqyclean_data, headers)

    def _clean_keys(self, keys):
        """ Given a list of keys, make them easier to read for plot labels
        """
        cats = OrderedDict()
        for k in keys:
            nice_name = re.sub("([a-z])([A-Z])","\g<1> \g<2>", k) # CamelCase > Camel Case
            nice_name = re.sub("([PS]E\d?)","\g<1> ", nice_name) # PE1Label > PE1 Label
            nice_name = re.sub("W([A-Z])","W \g<1>", nice_name) # WContam > W Contam
            nice_name = nice_name.replace('_', ' ') # tags_found > tags found
            nice_name = nice_name.title() # Title Case
            nice_name = nice_name.replace('Pe', 'PE').replace('Se', 'SE')
            nice_name = nice_name.replace('Tru SEq', 'TruSeq')
            cats[k] = { 'name': nice_name }
        return cats



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard QualityScoreDistribution """

import logging
import os
import re
from collections import OrderedDict

from multiqc.plots import linegraph
from .util import read_histogram

# Initialise the logger
log = logging.getLogger(__name__)

def parse_reports(self):
    """ Find Picard QualityScoreDistribution reports and parse their data """

    headers = ['QUALITY', 'COUNT_OF_Q']
    formats = [int, int]
    all_data = read_histogram(self, 'picard/quality_score_distribution', 'QualityScoreDistribution', headers, formats)

    if not all_data:
        return 0

    # Write parsed data to a file
    self.write_data_file(all_data, 'multiqc_picard_quality_score_distribution')

    # Plot the data and add section
    pconfig = {
        'id': 'picard_quality_score_distribution',
        'title': 'Picard: Base Quality Distribution',
        'ylab': 'Number of Bases',
        'xlab': 'Base Quality Score',
        'xDecimals': False,
        'tt_label': '<b>base quality{point.x}</b>: {point.y}',
        'ymin': 0,
    }

    lg = {}
    for s_name in all_data:
        lg[s_name] = OrderedDict((qual, data['COUNT_OF_Q']) for qual, data in all_data[s_name].items())

    self.add_section (
        name = 'Base Quality Distribution',
        anchor = 'picard-quality-score-distribution',
        description = 'Plot shows the count of each base quality score.',
        plot = linegraph.plot([lg], pconfig)
    )

    # Return the number of detected samples to the parent module
    return len(all_data)



#!/usr/bin/env python
"""
MultiQC is a tool to aggregate bioinformatics results across many samples into a single report. It is written in Python and contains modules for a large number of common bioinformatics tools.

You can install MultiQC from PyPI as follows::

    pip install multiqc

Then it's just a case of going to your analysis directory and running the script::

    multiqc .

MultiQC will scan the specified directory (:code:`'.'` is the current dir) and produce a report detailing whatever it finds.

The report is created in :code:`multiqc_report.html` by default. Tab-delimited data files are created in :code:`multiqc_data/` to give easy access for downstream processing.

For more detailed instructions, run :code:`multiqc -h`

See the MultiQC website for documentation and tutorial videos: http://multiqc.info

MultiQC was written by Phil Ewels (http://phil.ewels.co.uk) at SciLifeLab Sweden (http://www.scilifelab.se)
"""

from setuptools import setup, find_packages
import sys

version = '1.9'
dl_version = 'master' if 'dev' in version else 'v{}'.format(version)

print("""-----------------------------------
 Installing MultiQC version {}
-----------------------------------

""".format(version))

# Set version requirements according to what version of Python we're running
networkx_version = ''
numpy_version = ''
matplotlib_version = '>=2.1.1'
jinja2_version = '>=2.9'
markdown_version = ''
if sys.version_info[0:2] < (3, 6):
    # Lots of tools have dropped Python 3 support, so limit their versions
    matplotlib_version += ',<3.0.0'
    numpy_version = '<1.17'
    networkx_version = '<2.3'
    jinja2_version += ',<3.0'
    markdown_version = '<3.2'

install_requires = [
    'matplotlib' + matplotlib_version,
    'networkx' + networkx_version,
    'numpy' + numpy_version,
    'click',
    'coloredlogs',
    'future>0.14.0',
    'jinja2' + jinja2_version,
    'lzstring',
    'markdown' + markdown_version,
    'pyyaml>=4',
    'requests',
    'simplejson',
    'spectra>=0.0.10',
]

setup(
    name = 'multiqc',
    version = version,
    author = 'Phil Ewels',
    author_email = 'phil.ewels@scilifelab.se',
    description = "Create aggregate bioinformatics analysis reports across many samples and tools",
    long_description = __doc__,
    keywords = ['bioinformatics', 'biology', 'sequencing', 'NGS', 'next generation sequencing', 'quality control'],
    url = 'http://multiqc.info',
    download_url = 'https://github.com/ewels/MultiQC/tarball/{}'.format(dl_version),
    license = 'GPLv3',
    packages = find_packages(),
    include_package_data = True,
    zip_safe = False,
    install_requires = install_requires,
    entry_points = {
        "console_scripts": [
            "multiqc=multiqc.__main__:multiqc",
        ],
        'multiqc.modules.v1': [
            'adapterRemoval = multiqc.modules.adapterRemoval:MultiqcModule',
            'afterqc = multiqc.modules.afterqc:MultiqcModule',
            'bamtools = multiqc.modules.bamtools:MultiqcModule',
            'bbmap = multiqc.modules.bbmap:MultiqcModule',
            'bcftools = multiqc.modules.bcftools:MultiqcModule',
            'bcl2fastq = multiqc.modules.bcl2fastq:MultiqcModule',
            'biobambam2 = multiqc.modules.biobambam2:MultiqcModule',
            'biobloomtools = multiqc.modules.biobloomtools:MultiqcModule',
            'biscuit = multiqc.modules.biscuit:MultiqcModule',
            'bismark = multiqc.modules.bismark:MultiqcModule',
            'bowtie1 = multiqc.modules.bowtie1:MultiqcModule',
            'bowtie2 = multiqc.modules.bowtie2:MultiqcModule',
            'busco = multiqc.modules.busco:MultiqcModule',
            'clipandmerge = multiqc.modules.clipandmerge:MultiqcModule',
            'clusterflow = multiqc.modules.clusterflow:MultiqcModule',
            'conpair = multiqc.modules.conpair:MultiqcModule',
            'custom_content = multiqc.modules.custom_content:custom_module_classes', # special case
            'cutadapt = multiqc.modules.cutadapt:MultiqcModule',
            'damageprofiler = multiqc.modules.damageprofiler:MultiqcModule',
            'dedup = multiqc.modules.dedup:MultiqcModule',
            'deeptools = multiqc.modules.deeptools:MultiqcModule',
            'disambiguate = multiqc.modules.disambiguate:MultiqcModule',
            'dragen = multiqc.modules.dragen:MultiqcModule',
            'fastp = multiqc.modules.fastp:MultiqcModule',
            'fastq_screen = multiqc.modules.fastq_screen:MultiqcModule',
            'fastqc = multiqc.modules.fastqc:MultiqcModule',
            'featureCounts = multiqc.modules.featureCounts:MultiqcModule',
            'fgbio = multiqc.modules.fgbio:MultiqcModule',
            'flash = multiqc.modules.flash:MultiqcModule',
            'flexbar = multiqc.modules.flexbar:MultiqcModule',
            'gatk = multiqc.modules.gatk:MultiqcModule',
            'goleft_indexcov = multiqc.modules.goleft_indexcov:MultiqcModule',
            'happy = multiqc.modules.happy:MultiqcModule',
            'hicexplorer = multiqc.modules.hicexplorer:MultiqcModule',
            'hicpro = multiqc.modules.hicpro:MultiqcModule',
            'hicup = multiqc.modules.hicup:MultiqcModule',
            'hisat2 = multiqc.modules.hisat2:MultiqcModule',
            'homer = multiqc.modules.homer:MultiqcModule',
            'htseq = multiqc.modules.htseq:MultiqcModule',
            'interop = multiqc.modules.interop:MultiqcModule',
            'ivar = multiqc.modules.ivar:MultiqcModule',
            'jellyfish = multiqc.modules.jellyfish:MultiqcModule',
            'kaiju = multiqc.modules.kaiju:MultiqcModule',
            'kallisto = multiqc.modules.kallisto:MultiqcModule',
            'kat = multiqc.modules.kat:MultiqcModule',
            'kraken = multiqc.modules.kraken:MultiqcModule',
            'leehom = multiqc.modules.leehom:MultiqcModule',
            'longranger = multiqc.modules.longranger:MultiqcModule',
            'macs2 = multiqc.modules.macs2:MultiqcModule',
            'malt = multiqc.modules.malt:MultiqcModule',
            'methylQA = multiqc.modules.methylQA:MultiqcModule',
            'minionqc = multiqc.modules.minionqc:MultiqcModule',
            'mirtop = multiqc.modules.mirtop:MultiqcModule',
            'mirtrace = multiqc.modules.mirtrace:MultiqcModule',
            'mosdepth = multiqc.modules.mosdepth:MultiqcModule',
            'mtnucratio = multiqc.modules.mtnucratio:MultiqcModule',
            'multivcfanalyzer = multiqc.modules.multivcfanalyzer:MultiqcModule',
            'peddy = multiqc.modules.peddy:MultiqcModule',
            'phantompeakqualtools = multiqc.modules.phantompeakqualtools:MultiqcModule',
            'picard = multiqc.modules.picard:MultiqcModule',
            'preseq = multiqc.modules.preseq:MultiqcModule',
            'prokka = multiqc.modules.prokka:MultiqcModule',
            'pycoqc = multiqc.modules.pycoqc:MultiqcModule',
            'qorts = multiqc.modules.qorts:MultiqcModule',
            'qualimap = multiqc.modules.qualimap:MultiqcModule',
            'quast = multiqc.modules.quast:MultiqcModule',
            'rna_seqc = multiqc.modules.rna_seqc:MultiqcModule',
            'rockhopper = multiqc.modules.rockhopper:MultiqcModule',
            'rsem = multiqc.modules.rsem:MultiqcModule',
            'rseqc = multiqc.modules.rseqc:MultiqcModule',
            'salmon = multiqc.modules.salmon:MultiqcModule',
            'samblaster = multiqc.modules.samblaster:MultiqcModule',
            'samtools = multiqc.modules.samtools:MultiqcModule',
            'sargasso = multiqc.modules.sargasso:MultiqcModule',
            'seqyclean = multiqc.modules.seqyclean:MultiqcModule',
            'sexdeterrmine = multiqc.modules.sexdeterrmine:MultiqcModule',
            'sickle = multiqc.modules.sickle:MultiqcModule',
            'skewer = multiqc.modules.skewer:MultiqcModule',
            'slamdunk = multiqc.modules.slamdunk:MultiqcModule',
            'snpeff = multiqc.modules.snpeff:MultiqcModule',
            'snpsplit = multiqc.modules.snpsplit:MultiqcModule',
            'somalier = multiqc.modules.somalier:MultiqcModule',
            'sortmerna = multiqc.modules.sortmerna:MultiqcModule',
            'stacks = multiqc.modules.stacks:MultiqcModule',
            'star = multiqc.modules.star:MultiqcModule',
            'supernova = multiqc.modules.supernova:MultiqcModule',
            'theta2 = multiqc.modules.theta2:MultiqcModule',
            'tophat = multiqc.modules.tophat:MultiqcModule',
            'trimmomatic = multiqc.modules.trimmomatic:MultiqcModule',
            'varscan2 = multiqc.modules.varscan2:MultiqcModule',
            'vcftools = multiqc.modules.vcftools:MultiqcModule',
            'verifybamid = multiqc.modules.verifybamid:MultiqcModule'
        ],
        'multiqc.templates.v1': [
            'default = multiqc.templates.default',
            'default_dev = multiqc.templates.default_dev',
            'sections = multiqc.templates.sections',
            'simple = multiqc.templates.simple',
            'gathered = multiqc.templates.gathered',
            'geo = multiqc.templates.geo',
        ],
## See https://multiqc.info/docs/#multiqc-plugins for documentation
#       'multiqc.cli_options.v1': [
#           'my-new-option = myplugin.cli:new_option'
#       ],
#       'multiqc.hooks.v1': [
#           'before_config = myplugin.hooks:before_config',
#           'config_loaded = myplugin.hooks:config_loaded',
#           'execution_start = myplugin.hooks:execution_start',
#           'before_modules = myplugin.hooks:before_modules',
#           'after_modules = myplugin.hooks:after_modules',
#           'before_report_generation = myplugin.hooks:before_report_generation',
#           'execution_finish = myplugin.hooks:execution_finish',
#       ]
    },
    classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python',
        'Programming Language :: JavaScript',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Visualization',
    ],
)

print("""
--------------------------------
 MultiQC installation complete!
--------------------------------
For help in running MultiQC, please see the documentation available
at http://multiqc.info or run: multiqc --help
""")



#!/usr/bin/env python

""" MultiQC module to parse logs from Skewer """

from __future__ import print_function

from collections import OrderedDict
import logging
import re
from multiqc.plots import linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """ Skewer """

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Skewer', anchor='skewer',
                                            href="https://github.com/relipmoc/skewer",
                                            info="is an adapter trimming tool specially designed for processing next-generation sequencing (NGS) paired-end sequences.")

        self.skewer_data = dict()
        self.skewer_readlen_dist = dict()

        for f in self.find_log_files('skewer', filehandles=True):
            self.parse_skewer_log(f)

        # Filter to strip out ignored sample names
        self.skewer_data = self.ignore_samples(self.skewer_data)

        if len(self.skewer_data) == 0:
            raise UserWarning

        headers = OrderedDict()
        headers['pct_trimmed'] = {
            'title': '% Trimmed',
            'description': '% of reads trimmed',
            'scale': 'RdYlGn-rev',
            'max': 100,
            'min': 0,
            'suffix': '%',
        }

        self.general_stats_addcols(self.skewer_data, headers)

        # Write parsed report data to a file
        self.write_data_file(self.skewer_data, 'multiqc_skewer')
        self.write_data_file(self.skewer_readlen_dist, 'multiqc_skewer_readlen_dist')

        # set the value 0 for every x where a given sample doens't have a value
        all_x_values = []
        for s_name in self.skewer_readlen_dist:
            for xval in self.skewer_readlen_dist[s_name]:
                all_x_values.append(xval)

        for s_name in self.skewer_readlen_dist:
            for xval in all_x_values:
                if not xval in self.skewer_readlen_dist[s_name]:
                    self.skewer_readlen_dist[s_name][xval] = 0.0

            # After adding new elements, the ordereddict needs to be re-sorted
            items = self.skewer_readlen_dist[s_name]
            self.skewer_readlen_dist[s_name] = OrderedDict(sorted(items.items(), key=lambda x: int(x[0])))

        # add the histogram to the report
        self.add_readlen_dist_plot()

        log.info("Found {} reports".format(len(self.skewer_data)))

    def add_readlen_dist_plot(self):
        """ Generate plot HTML for read length distribution plot. """
        pconfig = {
            'id': 'skewer_read_length_histogram',
            'title': 'Skewer: Read Length Distribution after trimming',
            'xDecimals': False,
            'ylab': '% of Reads',
            'xlab': 'Read Length',
            'xmin': 0,
            'ymin': 0,
            'tt_label': '<b>{point.x}</b>: {point.y:.1f}%',
        }
        self.add_section(
            plot = linegraph.plot(self.skewer_readlen_dist, pconfig)
        )

    def parse_skewer_log(self, f):
        """ Go through log file looking for skewer output """
        fh = f['f']
        regexes = {
            'fq1': "Input file:\s+(.+)",
            'fq2': "Paired file:\s+(.+)",
            'r_processed': "(\d+) read|reads pairs? processed",
            'r_short_filtered': "(\d+) \(\s*\d+.\d+%\) short read",
            'r_empty_filtered': "(\d+) \(\s*\d+.\d+%\) empty read",
            'r_avail': "(\d+) \(\s*\d+.\d+%\) read",
            'r_trimmed': "(\d+) \(\s*\d+.\d+%\) trimmed read",
            'r_untrimmed': "(\d+) \(\s*\d+.\d+%\) untrimmed read"
        }
        regex_hist = "\s?(\d+)\s+(\d+)\s+(\d+.\d+)%"

        data = dict()
        for k, v in regexes.items():
            data[k] = 0
        data['fq1'] = None
        data['fq2'] = None
        readlen_dist = OrderedDict()

        for l in fh:
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    data[k] = match.group(1).replace(',', '')

            match = re.search(regex_hist, l)
            if match:
                read_length = int(match.group(1))
                pct_at_rl = float(match.group(3))
                readlen_dist[read_length] = pct_at_rl

        if data['fq1'] is not None:
            s_name = self.clean_s_name(data['fq1'], f['root'])
            if s_name in self.skewer_readlen_dist:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name)
            self.add_skewer_data(s_name, data, f)
            self.skewer_readlen_dist[s_name] = readlen_dist

        if data['fq2'] is not None:
            s_name = self.clean_s_name(data['fq1'], f['root'])
            if s_name in self.skewer_readlen_dist:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name)
            self.add_skewer_data(s_name, data, f)
            self.skewer_readlen_dist[s_name] = readlen_dist

    def add_skewer_data(self, s_name, data, f):
        stats = ['r_processed', 'r_short_filtered', 'r_empty_filtered', 'r_avail', 'r_trimmed', 'r_untrimmed']
        if s_name in self.skewer_data:
            log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
        self.skewer_data[s_name] = {}
        self.add_data_source(f, s_name)
        for k in stats:
            self.skewer_data[s_name][k] = int(data[k])

        self.skewer_data[s_name]['pct_avail'] = 100.0 * float(data['r_avail']) / float(data['r_processed'])
        self.skewer_data[s_name]['pct_trimmed'] = 100.0 * float(data['r_trimmed']) / float(data['r_avail'])
        self.skewer_data[s_name]['pct_untrimmed'] = 100.0 * float(data['r_untrimmed']) / float(data['r_avail'])



#!/usr/bin/env python

""" MultiQC module to parse output from Cluster Flow """

from __future__ import print_function
from collections import OrderedDict
import datetime
import logging
import re
import os
import time

from multiqc.plots import table
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    Cluster Flow module class, parses run logs.
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Cluster Flow', anchor='clusterflow',
        href='http://clusterflow.io',
        info="is a simple and flexible bioinformatics pipeline tool.")

        # Find and load any Cluster Flow reports
        self.clusterflow_commands = dict()
        self.clusterflow_runfiles = dict()

        for f in self.find_log_files('clusterflow/logs', filehandles=True):
            self.parse_clusterflow_logs(f)
            self.add_data_source(f, 'log')
        for f in self.find_log_files('clusterflow/runfiles', filehandles=True):
            parsed_data = self.parse_clusterflow_runfiles(f)
            if parsed_data is not None:
                self.clusterflow_runfiles[f['s_name']] = parsed_data
                self.add_data_source(f, 'runfile')

        # Filters to strip out ignored sample names
        self.clusterflow_commands = self.ignore_samples(self.clusterflow_commands)
        self.clusterflow_runfiles = self.ignore_samples(self.clusterflow_runfiles)

        if len(self.clusterflow_commands) == 0 and len(self.clusterflow_runfiles) == 0:
            raise UserWarning

        # Count pipelines
        num_log_pipelines = len(self.clusterflow_commands)
        num_runfile_pipelines = len(set(d.get('pipeline_id', 'unknown') for d in self.clusterflow_runfiles.values()))
        log.info("Found {} pipelines".format(max(num_log_pipelines, num_runfile_pipelines)))
        log.debug("Found {} log pipelines".format(num_log_pipelines))
        log.debug("Found {} runfile pipelines".format(num_runfile_pipelines))

        # Pipeline Info
        if len(self.clusterflow_runfiles) > 0:
            self.clusterflow_pipelines_section()

        # Commands
        if len(self.clusterflow_commands) > 0:
            self.clusterflow_commands_table()


    def parse_clusterflow_logs(self, f):
        """ Parse Clusterflow logs """
        module = None
        job_id = None
        pipeline_id = None
        for l in f['f']:

            # Get pipeline ID
            module_r = re.match(r'Module:\s+(.+)$', l)
            if module_r:
                module = module_r.group(1)
            job_id_r = re.match(r'Job ID:\s+(.+)$', l)
            if job_id_r:
                job_id = job_id_r.group(1)
                if module is not None:
                    pipeline_r = re.match(r"(cf_.+)_"+re.escape(module)+r"_\d+$", job_id)
                    if pipeline_r:
                        pipeline_id = pipeline_r.group(1)

            # Get commands that have been run
            if l.startswith('###CFCMD'):
                if pipeline_id is None:
                    pipeline_id = 'unknown'
                if pipeline_id not in self.clusterflow_commands.keys():
                    self.clusterflow_commands[pipeline_id] = list()
                self.clusterflow_commands[pipeline_id].append(l[8:])


    def clusterflow_commands_table (self):
        """ Make a table of the Cluster Flow commands """

        # I wrote this when I was tired. Sorry if it's incomprehensible.

        desc = '''Every Cluster Flow run will have many different commands.
            MultiQC splits these by whitespace, collects by the tool name
            and shows the first command found. Any terms not found in <em>all</em> subsequent
            calls are replaced with <code>[variable]</code>
            <em>(typically input and ouput filenames)</em>. Each column is for one Cluster Flow run.'''

        # Loop through pipelines
        tool_cmds = OrderedDict()
        headers = dict()
        for pipeline_id, commands in self.clusterflow_commands.items():
            headers[pipeline_id] = {'scale': False}
            self.var_html = '<span style="background-color:#dedede; color:#999;">[variable]</span>'
            tool_cmd_parts = OrderedDict()
            for cmd in commands:
                s = cmd.split()
                tool = self._guess_cmd_name(s)
                if tool not in tool_cmd_parts.keys():
                    tool_cmd_parts[tool] = list()
                tool_cmd_parts[tool].append(s)


            for tool, cmds in tool_cmd_parts.items():
                cons_cmd = self._replace_variable_chunks(cmds)
                # Try again with first two blocks if all variable
                variable_count = cons_cmd.count(self.var_html)
                if variable_count == len(cmds[0]) - 1 and len(cmds[0]) > 2:
                    for subcmd in set([x[1] for x in cmds]):
                        sub_cons_cmd = self._replace_variable_chunks([cmd for cmd in cmds if cmd[1] == subcmd])
                        tool = "{} {}".format(tool, subcmd)
                        if tool not in tool_cmds:
                            tool_cmds[tool] = dict()
                        tool_cmds[tool][pipeline_id] = '<code style="white-space:nowrap;">{}</code>'.format(" ".join(sub_cons_cmd) )
                else:
                    if tool not in tool_cmds:
                        tool_cmds[tool] = dict()
                    tool_cmds[tool][pipeline_id] = '<code style="white-space:nowrap;">{}</code>'.format(" ".join(cons_cmd) )

        table_config = {
            'namespace': 'Cluster Flow',
            'id': 'clusterflow-commands-table',
            'table_title': 'Cluster Flow Commands',
            'col1_header': 'Tool',
            'sortRows': False,
            'no_beeswarm': True
        }
        self.add_section (
            name = 'Commands',
            anchor = 'clusterflow-commands',
            description = desc,
            plot = table.plot(tool_cmds, headers, table_config)
        )


    def _replace_variable_chunks(self, cmds):
        """ List through a list of command chunks. Return a single list
        with any variable bits blanked out. """

        cons_cmd = None
        while cons_cmd is None:
            for cmd in cmds:
                if cons_cmd is None:
                    cons_cmd = cmd[:]
                else:
                    for idx, s in enumerate(cons_cmd):
                        if s not in cmd:
                            cons_cmd[idx] = self.var_html
        return cons_cmd

    def _guess_cmd_name(self, cmd):
        """ Manually guess some known command names, where we can
        do a better job than the automatic parsing. """

        # zcat to bowtie
        if cmd[0] == 'zcat' and 'bowtie' in cmd:
            return 'bowtie'
        # samtools
        if cmd[0] == 'samtools':
            return ' '.join(cmd[0:2])
        # java (eg. picard)
        if cmd[0] == 'java':
            jars = [s for s in cmd if '.jar' in s]
            return os.path.basename(jars[0].replace('.jar', ''))
        return cmd[0]


    def parse_clusterflow_runfiles(self, f):
        """ Parse run files generated by Cluster Flow.
        Currently gets pipeline IDs and associated steps."""

        data = dict()
        in_comment = False
        seen_pipeline = False
        cf_file = False
        for l in f['f']:
            l = l.rstrip()
            # Check that this is from Cluster Flow
            if 'Cluster Flow' in l:
                cf_file = True
            # Header
            if l.startswith('Pipeline: '):
                data['pipeline_name'] = l[10:]
            if l.startswith('Pipeline ID: '):
                data['pipeline_id'] = l[13:]
            if l.startswith('Created at '):
                data['pipeline_start'] = l[11:]
            # Config settings
            if l.startswith('@'):
                s = l.split(None, 1)
                key = s[0].replace('@', '').strip()
                try:
                    data[key] = "\t".join(s[1:])
                except IndexError:
                    data[key] = True
            # Comments
            if l.startswith('/*'):
                in_comment = True
            if l.startswith('*/'):
                in_comment = False
            if in_comment:
                if 'comment' not in data:
                    data['comment'] = ''
                data['comment'] += l+"\n"
            # Pipeline steps
            if l.strip().startswith('#'):
                if 'pipeline_steps' not in data:
                    data['pipeline_steps'] = []
                data['pipeline_steps'].append(l)
                seen_pipeline = True
            # Step output files
            elif seen_pipeline:
                s = l.split("\t")
                if len(s) > 1:
                    if 'files' not in data:
                        data['files'] = OrderedDict()
                    if s[0] not in data['files']:
                        data['files'][s[0]] = []
                    data['files'][s[0]].append(s[1:])
        # Parse the start date
        dt = None
        if 'pipeline_id' in data:
            s = data['pipeline_id'].split('_')
            dt = datetime.datetime.fromtimestamp(int(s[-1]))
        elif 'pipeline_start' in data:
            dt_r = re.match(r'(\d{2}):(\d{2}), (\d{2})-(\d{2})-(\d{4})', data['pipeline_start'])
            if dt_r:
                dt = datetime.datetime(
                    int(dt_r.group(5)), # year
                    int(dt_r.group(4)), # month
                    int(dt_r.group(3)), # day
                    int(dt_r.group(1)), # hour
                    int(dt_r.group(2))  # minute
                )

        # Not a Cluster Flow file (eg. Nextflow .run file)
        if not cf_file:
            return None

        if dt is not None:
            data['pipeline_start_dateparts'] = {
                'year':        dt.year,
                'month':       dt.month,
                'day':         dt.day,
                'hour':        dt.hour,
                'minute':      dt.minute,
                'second':      dt.second,
                'microsecond': dt.microsecond,
                'timestamp':   time.mktime(dt.timetuple())
            }
        # Cluster Flow v0.4 and before did not print the pipeline ID in run files
        # Try to guess - will be wrong as no microsecond info, but hopefully unique
        # and reproducible for other run files
        if 'pipeline_id' not in data:
            if 'pipeline_name' in data and 'pipeline_start_dateparts' in data:
                log.debug('Trying to guess pipeline ID for file "{}"'.format(f['fn']))
                data['pipeline_id'] = 'cf_{}_{}'.format(data['pipeline_name'], data['pipeline_start_dateparts']['timestamp'])
        return data

    def clusterflow_pipelines_section(self):
        """ Generate HTML for section about pipelines, generated from
        information parsed from run files. """
        data = dict()
        pids_guessed = ''
        for f,d in self.clusterflow_runfiles.items():
            pid = d.get('pipeline_id', 'unknown')
            if d.get('pipeline_id_guess', False) is True:
                pid += '*'
                pids_guessed = ' Project IDs with an asterisk may be inaccurate.'
            # Count the number of files going into the first module
            num_starting_files = 0
            for step_name, files in d.get('files',{}).items():
                if step_name.startswith('start'):
                    num_starting_files += len(files)
            # Reformat the date so that column sorting works nicely
            if 'pipeline_start_dateparts' in d:
                dt = d['pipeline_start_dateparts']
                d['pipeline_start'] = '{}-{:02d}-{:02d} {:02d}:{:02d}'.format(dt['year'], dt['month'], dt['day'], dt['hour'], dt['minute'])
            if pid not in data:
                data[pid] = d
                data[pid]['num_starting_files'] = int(num_starting_files)
            else:
                data[pid]['num_starting_files'] += int(num_starting_files)

        headers = OrderedDict()
        headers['pipeline_name'] = {'title': 'Pipeline Name'}
        headers['pipeline_start'] = {'title': 'Date Started', 'description': 'Date and time that pipeline was started (YYYY-MM-DD HH:SS)'}
        headers['genome'] = {'title': 'Genome ID', 'description': 'ID of reference genome used'}
        headers['num_starting_files'] = {'title': '# Starting Files', 'format': '{:,.0f}', 'description': 'Number of input files at start of pipeline run.'}
        table_config = {
            'namespace': 'Cluster Flow',
            'id': 'clusterflow-pipelines-table',
            'table_title': 'Cluster Flow Pipelines',
            'col1_header': 'Pipeline ID',
            'no_beeswarm': True,
            'save_file': True
        }
        self.add_section (
            name = 'Pipelines',
            anchor = 'clusterflow-pipelines',
            description = 'Information about pipelines is parsed from <code>*.run</code> files. {}'.format(pids_guessed),
            plot = table.plot(data, headers, table_config),
            content = self.clusterflow_pipelines_printout()
        )


    def clusterflow_pipelines_printout(self):
        """ Print the steps used in each Cluster Flow pipeline """
        data = dict()
        html = ''
        for f,d in self.clusterflow_runfiles.items():
            pid = d.get('pipeline_id', 'unknown')
            data[pid] = [
                d.get('pipeline_name'),
                "\n".join(d.get('pipeline_steps', []))
            ]
        for pid, d in data.items():
            html += '''
                <div class="panel panel-default">
                    <div class="panel-heading"><h3 class="panel-title">Pipeline Steps: {} (<code>{}</code>)</h3></div>
                    <pre class="panel-body" style="border:0; background-color:transparent; padding:0 15px; margin:0; color:#666; font-size:90%;">{}</pre>
                </div>
                '''.format(pid, d[0], d[1])
        return html



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard GcBiasMetrics """

import logging
import os
import re

from multiqc.plots import linegraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(self):
    """ Find Picard GcBiasMetrics reports and parse their data """

    # Set up vars
    self.picard_GCbias_data = dict()
    self.picard_GCbiasSummary_data = dict()

    # Go through logs and find Metrics
    for f in self.find_log_files('picard/gcbias', filehandles=True):
        s_name = None
        gc_col = None
        cov_col = None
        for l in f['f']:
            # New log starting
            if 'GcBiasMetrics' in l and 'INPUT' in l:
                s_name = None

                # Pull sample name from input
                fn_search = re.search(r"INPUT(?:=|\s+)(\[?[^\s]+\]?)", l, flags=re.IGNORECASE)
                if fn_search:
                    s_name = os.path.basename(fn_search.group(1).strip('[]'))
                    s_name = self.clean_s_name(s_name, f['root'])

            if s_name is not None:
                if gc_col is not None and cov_col is not None :
                    try:
                        # Note that GC isn't always the first column.
                        s = l.strip("\n").split("\t")
                        self.picard_GCbias_data[s_name][ int(s[gc_col]) ] = float(s[cov_col])
                    except IndexError:
                        s_name = None
                        gc_col = None
                        cov_col = None

                if 'GcBiasDetailMetrics' in l and '## METRICS CLASS' in l:
                    if s_name in self.picard_GCbias_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                    self.add_data_source(f, s_name, section='GcBiasDetailMetrics')
                    self.picard_GCbias_data[s_name] = dict()
                    # Get header - find columns with the data we want
                    l = f['f'].readline()
                    s = l.strip("\n").split("\t")
                    gc_col = s.index('GC')
                    cov_col = s.index('NORMALIZED_COVERAGE')

                if 'GcBiasSummaryMetrics' in l and '## METRICS CLASS' in l:
                    if s_name in self.picard_GCbiasSummary_data:
                        log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
                    self.add_data_source(f, s_name, section='GcBiasSummaryMetrics')
                    self.picard_GCbiasSummary_data[s_name] = dict()

                    keys = f['f'].readline().rstrip("\n").split("\t")
                    vals = f['f'].readline().rstrip("\n").split("\t")
                    for i, k in enumerate(keys):
                        try:
                            self.picard_GCbiasSummary_data[s_name][k] = float(vals[i])
                        except ValueError:
                            self.picard_GCbiasSummary_data[s_name][k] = vals[i]


        for s_name in list(self.picard_GCbias_data.keys()):
            if len(self.picard_GCbias_data[s_name]) == 0:
                self.picard_GCbias_data.pop(s_name, None)
                log.debug("Removing {} as no data parsed".format(s_name))

        for s_name in list(self.picard_GCbiasSummary_data.keys()):
            if len(self.picard_GCbiasSummary_data[s_name]) == 0:
                self.picard_GCbiasSummary_data.pop(s_name, None)
                log.debug("Removing {} as no data parsed".format(s_name))


    # Filter to strip out ignored sample names
    self.picard_GCbias_data = self.ignore_samples(self.picard_GCbias_data)

    if len(self.picard_GCbias_data) > 0:

        # Plot the graph

        pconfig = {
            'id': 'picard_gcbias_plot',
            'title': 'Picard: GC Coverage Bias',
            'ylab': 'Normalized Coverage',
            'xlab': '% GC',
            'xmin': 0,
            'xmax': 100,
            'xDecimals': False,
            'ymin': 0,
            'yCeiling': 10,
            'tt_label': '<b>{point.x} %GC</b>: {point.y:.2f}',
            'yPlotLines': [
                {'value': 1, 'color': '#999999', 'width': 2, 'dashStyle': 'LongDash'},
            ]
        }
        self.add_section (
            name = 'GC Coverage Bias',
            anchor = 'picard-gcbias',
            description = 'This plot shows bias in coverage across regions of the genome with varying GC content.'\
                ' A perfect library would be a flat line at <code>y = 1</code>.',
            plot = linegraph.plot(self.picard_GCbias_data, pconfig)
        )

    if len(self.picard_GCbiasSummary_data) > 0:
        # Write parsed summary data to a file
        self.write_data_file(self.picard_GCbiasSummary_data, 'multiqc_picard_gcbias')


    # Return the number of detected samples to the parent module
    return len(self.picard_GCbias_data)



#!/usr/bin/env python

""" MultiQC module to parse output from Samblaster """

from __future__ import print_function
import os
from collections import OrderedDict
import logging
import re
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):
    """ Samblaster """

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='Samblaster', anchor='samblaster',
                                            href="https://github.com/GregoryFaust/samblaster",
                                            info="is a tool to mark duplicates and extract discordant and split reads from sam files.")

        self.samblaster_data = dict()
        for f in self.find_log_files('samblaster', filehandles=True):
            self.parse_samblaster(f)

        # Filter to strip out ignored sample names
        self.samblaster_data = self.ignore_samples(self.samblaster_data)

        if len(self.samblaster_data) == 0:
            raise UserWarning

        headers = OrderedDict()
        headers['pct_dups'] = {
            'title': '% Dups',
            'description': 'Percent Duplication',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'OrRd'
        }

        self.general_stats_addcols(self.samblaster_data, headers)

        # Write parsed report data to a file
        self.write_data_file(self.samblaster_data, 'multiqc_samblaster')

        log.info("Found {} reports".format(len(self.samblaster_data)))

        self.add_barplot()

    def add_barplot(self):
        """ Generate the Samblaster bar plot. """
        cats = OrderedDict()
        cats['n_nondups'] = {'name': 'Non-duplicates'}
        cats['n_dups'] = {'name': 'Duplicates'}

        pconfig = {
            'id': 'samblaster_duplicates',
            'title': 'Samblaster: Number of duplicate reads',
            'ylab': 'Number of reads'
        }
        self.add_section( plot = bargraph.plot(self.samblaster_data, cats, pconfig) )

    def parse_samblaster(self, f):
        """ Go through log file looking for samblaster output.
        If the
        Grab the name from the RG tag of the preceding bwa command """

        # Should capture the following:
        # samblaster: Marked     1134898 of   43791982 (2.592%) total read ids as duplicates using 753336k \
        # memory in 1M1S(60.884S) CPU seconds and 3M53S(233S) wall time.
        dups_regex = r"samblaster: (Removed|Marked)\s+(\d+)\s+of\s+(\d+) \((\d+.\d+)%\)\s*(total)?\s*read ids as duplicates"

        input_file_regex = "samblaster: Opening (\S+) for read."
        rgtag_name_regex = "\\\\tID:(\S*?)\\\\t"
        data = {}
        s_name = None
        fh = f['f']
        for l in fh:
            # try to find name from RG-tag. If bwa mem is used upstream samblaster with pipes, then the bwa mem command
            # including the read group will be written in the log
            match = re.search(rgtag_name_regex, l)
            if match:
                s_name = self.clean_s_name( match.group(1), f['root'])

            # try to find name from the input file name, if used
            match = re.search(input_file_regex, l)
            if match:
                basefn = os.path.basename(match.group(1))
                fname, ext = os.path.splitext(basefn)
                # if it's stdin, then try bwa RG-tag instead
                if fname != 'stdin':
                    s_name = self.clean_s_name( fname, f['root'])

            match = re.search(dups_regex, l)
            if match:
                data['n_dups'] = int(match.group(2))
                data['n_tot'] = int(match.group(3))
                data['n_nondups'] = data['n_tot'] - data['n_dups']
                data['pct_dups'] = float(match.group(4))

        if s_name is None:
            s_name = f['s_name']

        if len(data) > 0:
            if s_name in self.samblaster_data:
                log.debug("Duplicate sample name found in {}! Overwriting: {}".format(f['fn'], s_name))
            self.add_data_source(f, s_name)
            self.samblaster_data[s_name] = data



#!/usr/bin/env python
from __future__ import print_function

import re
from collections import defaultdict
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.plots import linegraph

# Initialise the logger
import logging
log = logging.getLogger(__name__)


MIN_CNT_TO_SHOW_ON_PLOT = 5


class DragenFragmentLength(BaseMultiqcModule):
    def add_fragment_length_hist(self):
        data_by_rg_by_sample = defaultdict(dict)

        for f in self.find_log_files('dragen/fragment_length_hist'):
            data_by_rg = parse_fragment_length_hist_file(f)
            if f['s_name'] in data_by_rg_by_sample:
                log.debug('Duplicate sample name found! Overwriting: {}'.format(f['s_name']))
            self.add_data_source(f, section='stats')

            for rg, data in data_by_rg.items():
                if any(rg in d_rg for sn, d_rg in data_by_rg_by_sample.items()):
                    log.debug('Duplicate read group name {} found for {}! Overwriting'.format(rg, f['s_name']))
            data_by_rg_by_sample[f['s_name']].update(data_by_rg)

        # Filter to strip out ignored sample names:
        data_by_rg_by_sample = self.ignore_samples(data_by_rg_by_sample)
        if not data_by_rg_by_sample:
            return set()

        # Merging all data
        data_by_rg = {}
        for sn, d_rg in data_by_rg_by_sample.items():
            for rg, d in d_rg.items():
                if rg in d_rg:
                    rg = rg + ' (' + sn + ')'
                data_by_rg[rg] = d

        smooth_points = 300
        self.add_section(
            name='Fragment length hist',
            anchor='dragen-fragment-length-histogram',
            description="""
            Distribution of estimated fragment lengths of mapped reads per read group. 
            Only points supported by at least {} reads are shown to prevent long flat tail. 
            The plot is also smoothed down to showing 300 points on the X axis to reduce noise.
            """.format(MIN_CNT_TO_SHOW_ON_PLOT),
            plot=linegraph.plot(data_by_rg, {
                'id': 'dragen_fragment_length',
                'title': 'Dragen: Fragment length hist',
                'ylab': 'Number of reads',
                'xlab': 'Fragment length (bp)',
                'ymin': 0,
                'xmin': 0,
                'tt_label': '<b>{point.x} bp</b>: {point.y} reads',
                'smooth_points': smooth_points,
            })
        )
        return data_by_rg_by_sample.keys()


def parse_fragment_length_hist_file(f):
    """
    T_SRR7890936_50pc.fragment_length_hist.csv

    #Sample: N_SRR7890889
    FragmentLength,Count
    36,1
    37,0
    38,0
    39,0
    40,0
    41,1
    ...
    39203,0
    39204,0
    39205,1
    #Sample: T_SRR7890936_50pc
    FragmentLength,Count
    53,2
    54,0
    ...
    39316,0
    39317,1
    """

    f['s_name'] = re.search(r'(.*).fragment_length_hist.csv', f['fn']).group(1)

    data_by_rg = defaultdict(dict)

    read_group = None
    for line in f['f'].splitlines():
        if line.startswith('#Sample'):
            read_group = line.split('#Sample: ')[1]
        else:
            assert read_group is not None
            frag_len, cnt = line.split(',')
            try:
                frag_len = int(frag_len)
                cnt = int(cnt)
            except ValueError:
                assert line == 'FragmentLength,Count', line
            else:
                if cnt >= MIN_CNT_TO_SHOW_ON_PLOT:  # to prevent long flat tail
                    data_by_rg[read_group][frag_len] = cnt

    return data_by_rg



#!/usr/bin/env python
## Nicolas Servant
## April 2018

""" MultiQC module to parse output from HiC-Pro """

from __future__ import print_function
from collections import OrderedDict
import os.path
import logging

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ HiC-Pro module, parses log and stats files saved by HiC-Pro. """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='HiC-Pro', anchor='hicpro',
        href='https://github.com/nservant/HiC-Pro',
        info="is an efficient and flexible pipeline for Hi-C data processing. The MultiQC module is supported since HiC-Pro v2.11.0.")

        # Find and load any HiC-Pro summary reports
        self.hicpro_data = dict()
        for k in ['mmapstat','mpairstat','mergestat','mRSstat','assplit']:
            for f in self.find_log_files('hicpro/{}'.format(k)):
                self.parse_hicpro_stats(f, k)
        
        # Update current statistics
        for s_name in self.hicpro_data:
            data = self.hicpro_data[s_name]
            data['duplicates'] = data['valid_interaction'] - data['valid_interaction_rmdup']
            data['percent_duplicates'] = float(data['duplicates']) / float(data['valid_interaction']) * 100.0
            data['percent_mapped_R1'] = float(data['mapped_R1']) / float(data['total_R1']) * 100.0
            data['percent_mapped_R2'] = float(data['mapped_R2']) / float(data['total_R2']) * 100.0
            data['paired_reads'] = int(data['Reported_pairs'])
            data['percent_paired_reads'] = float(data['Reported_pairs']) / float(data['total_R1']) * 100.0
            data['percent_valid'] = float(data['valid_interaction']) / float(data['total_R1']) * 100.0
            data['Failed_To_Align_Read_R1'] = int(data['total_R1']) - int(data['mapped_R1'])
            data['Failed_To_Align_Read_R2'] = int(data['total_R2']) - int(data['mapped_R2'])
            
            try:
                data['valid_pairs_off_target'] = int(data['valid_interaction']) - int(data['valid_pairs_on_target'])
            except KeyError:
                pass
            
        # Filter to strip out ignored sample names
        self.hicpro_data = self.ignore_samples(self.hicpro_data)

        if len(self.hicpro_data) == 0:
            raise UserWarning

        log.info("Found {} HiC-Pro reports".format(len(self.hicpro_data)))
        
        # Write parsed data to a file
        self.write_data_file(self.hicpro_data, 'multiqc_hicpro')

        # Basic Stats Table
        self.hicpro_stats_table()

        # Report sections
        self.add_section (
            name = 'Read Mapping',
            anchor = 'hicpro-mapping',
            description = 'Alignment of reads in single-end mode.',
            helptext = '''
            HiC-Pro uses a two steps mapping strategy. End-to-end mapping is first
            performed (*Full reads Alignments*). Unmapped reads are then trimmed
            at their ligation site, and the 5\' end is re-aligned on the reference
            genome (*Trimmed Reads Alignments*)''',
            plot = self.hicpro_mapping_chart()
        )

        self.add_section (
            name = 'Read Pairing',
            anchor = 'hicpro-pairing',
            description = 'Pairing of single-end mapping.',
            helptext = '''
            HiC-Pro combines aligned reads as pairs. Singleton, low quality pairs or 
            pairs involving multi-mapped reads are usually discarded. Note that the filtering 
            at pairing level can change accrding to the parameters used.''',
            plot = self.hicpro_pairing_chart()
        )
        
        self.add_section (
            name = 'Read Pair Filtering',
            anchor = 'hicpro-filtering',
            description = 'Filtering of read pairs.',
            helptext = '''
            Aligned read pairs are filtered to select valid 3C products from two 
            different restriction fragments. Read pairs coming from the same fragments, 
            such as self-ligation or unligated (danging-end) fragments, are discarded.
            Ligation products involving neighboring restriction fragment (religation) are also discarded.
            Filtered pairs correspond to ligation products discarded based on the range of insert/fragment sizes defined
            in the analysis. Finally, as the ligation should be a random process, valid read pairs from all 
            orientations (R=Reverse, F=forward) are expected to be observed in the same proportion.''',
            plot = self.hicpro_filtering_chart()
        )

        self.add_section (
            name = 'Contact Statistics',
            anchor = 'hicpro-rmdup',
            description = 'Contacts statistics after duplicates removal.',
            helptext = '''
            Description of contact frequency after duplicates removal. Intra-chromosomal 
            (*cis*) interaction are expected to be more frequent than inter-chromosomal contacts (*trans*)''',
            plot = self.hicpro_contact_chart()
        )

        allele_plot = self.hicpro_as_chart()
        if allele_plot:
            self.add_section (
                name = 'Allele-specific analysis',
                anchor = 'hicpro-asan',
                description = 'Assignment of valid interactions (afer duplicates removal) to parental alleles.',
                helptext = '''
                Description of allele-specific contacts. Valid interactions (0-1 and 1-1, resp. 0-2, 2-2) 
                are used to build the genome1 (resp. genome2) parental maps.''',
                plot = allele_plot
            )
            
        capture_plot = self.hicpro_capture_chart()
        if capture_plot:
            self.add_section (
                name = 'Capture analysis',
                anchor = 'hicpro-cap',
                description = 'Selection of interactions overlaping the targeted region(s).',
                helptext = '''
                Description of capture efficiency. Valid interactions with either two (capture-capture) or 
                one (capture-reporter) interactors overlapping with the target(s) are reported.''',
                plot = capture_plot
            )

    def parse_hicpro_stats(self, f, rsection):
        """ Parse a HiC-Pro stat file """
        s_name = self.clean_s_name(os.path.basename(f['root']), os.path.dirname(f['root']))

        if s_name not in self.hicpro_data.keys():
            self.hicpro_data[s_name] = {}

        self.add_data_source(f, s_name, section=rsection)
        for l in f['f'].splitlines():
            if not l.startswith('#'):
                s = l.split("\t")
                if s[0] in self.hicpro_data[s_name]:
                    log.debug("Duplicated keys found! Overwriting: {}".format(s[0]))
                self.hicpro_data[s_name][s[0]] = int(s[1])

    def hicpro_stats_table(self):
         """ Add HiC-Pro stats to the general stats table """
         headers = OrderedDict()

         headers['percent_duplicates'] = {
             'title': '% Duplicates',
             'description': 'Percent of duplicated valid pairs (%)',
             'max': 100,
             'min': 0,
             'suffix': '%',
             'scale': 'YlOrRd',
             'hidden': True
         }
            
         headers['valid_interaction_rmdup'] = {
             'title': '{} Valid Pairs Unique'.format(config.read_count_prefix),
             'description': 'Number of valid pairs after duplicates removal ({})'.format(config.read_count_desc),
             'min': 0,
             'scale': 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key': 'read_count',
         }
                 
         headers['percent_valid'] = {
             'title': '% Valid Pairs',
             'description': 'Percentage of valid pairs over reported ones (%)',
             'max': 100,
             'min': 0,
             'suffix': '%',
             'scale': 'YlGn',
             'hidden': True
         }

         headers['valid_interaction'] = {
             'title': '{} Valid Pairs'.format(config.read_count_prefix),
             'description': 'Number of valid pairs ({})'.format(config.read_count_desc),
             'min': 0,
             'scale': 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key': 'read_count',
             'hidden': True
         }
 
         headers['percent_paired_reads'] = {
             'title': '% Reported',
             'description': 'Percentage of paired reads (%) passing the mapping filters',
             'max': 100,
             'min': 0,
             'suffix': '%',
             'scale': 'YlGn',
             'hidden': True
         }

         headers['paired_reads'] = {
             'title': 'Reported Read Pairs',
             'description' : 'Total number of read pairs ({}) passing the mapping filters'.format(config.read_count_desc),
             'min' : '0',
             'scale' : 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key' : 'read_count',
         }

         headers['percent_mapped_R2'] = {
             'title': '% Aligned [R2]',
             'description': 'Percentage of aligned reads [R2] (%)',
             'max': 100,
             'min': 0,
             'suffix': '%',
             'scale': 'YlGn',
             'hidden': True
         }

         headers['mapped_R2'] = {
             'title': 'Aligned [R2]',
             'description' : 'Total number of aligned reads [R2] ({})'.format(config.read_count_desc),
             'min' : '0',
             'scale' : 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key' : 'read_count',
             'hidden': True
         }

         headers['percent_mapped_R1'] = {
             'title': '% Aligned [R1]',
             'description': 'Percentage of aligned reads [R1] (%)',
             'max': 100,
             'min': 0,
             'suffix': '%',
             'scale': 'YlGn',
             'hidden': True
         }

         headers['mapped_R1'] = {
             'title': 'Aligned [R1]',
             'description' : 'Total number of aligned reads [R1] ({})'.format(config.read_count_desc),
             'min' : '0',
             'scale' : 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key' : 'read_count',
             'hidden': True
         }
         
         headers['total_R1'] = {
             'title': 'Total',
             'description' : 'Total Number of Read Pairs',
             'min' : '0',
             'scale' : 'RdYlBu',
             'modify': lambda x: x * config.read_count_multiplier,
             'shared_key' : 'read_count',
         }

         self.general_stats_addcols(self.hicpro_data, headers, 'HiC-Pro')


    def hicpro_mapping_chart (self):
        """ Generate the HiC-Pro Aligned reads plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Full_Alignments_Read']   = { 'color': '#005ce6', 'name': 'Full reads Alignments' }
        keys['Trimmed_Alignments_Read'] = { 'color': '#3385ff', 'name': 'Trimmed reads Alignments' }
        keys['Failed_To_Align_Read']     = { 'color': '#a9a2a2', 'name': 'Failed To Align' }
                               
        data = [{},{}]
        for s_name in self.hicpro_data:
            for r in [1,2]:
                data[r-1]['{} [R{}]'.format(s_name, r)] = {
                    'Full_Alignments_Read': self.hicpro_data[s_name]['global_R{}'.format(r)],
                    'Trimmed_Alignments_Read': self.hicpro_data[s_name]['local_R{}'.format(r)],
                    'Failed_To_Align_Read': int(self.hicpro_data[s_name]['total_R{}'.format(r)]) - int(self.hicpro_data[s_name]['mapped_R{}'.format(r)])
                }
        
        # Config for the plot
        config = {
            'id': 'hicpro_mapping_stats_plot',
            'title': 'HiC-Pro: Mapping Statistics',
            'ylab': '# Reads',
            'ylab': '# Reads: Read 1',
            'data_labels': [
                {'name': 'Read 1', 'ylab': '# Reads: Read 1'},
                {'name': 'Read 2', 'ylab': '# Reads: Read 2'}
            ]
        }

        return bargraph.plot(data, [keys, keys], config)

    def hicpro_pairing_chart (self):
        """ Generate Pairing chart """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Unique_paired_alignments'] = { 'color': '#005ce6', 'name': 'Uniquely Aligned' }
        keys['Low_qual_pairs'] = { 'color': '#b97b35', 'name': 'Low Quality' }
        keys['Pairs_with_singleton'] = { 'color': '#ff9933', 'name': 'Singleton' }
        keys['Multiple_pairs_alignments'] = { 'color': '#e67300', 'name': 'Multi Aligned' }
        keys['Unmapped_airs'] = { 'color': '#a9a2a2', 'name': 'Failed To Align' }
    
        # Config for the plot
        config = {
            'id': 'hicpro_pairing_stats_plot',
            'title': 'HiC-Pro: Pairing Statistics',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.hicpro_data, keys, config)

        
    def hicpro_filtering_chart (self):
        """ Generate the HiC-Pro filtering plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['Valid_interaction_pairs_FF'] = { 'color': '#ccddff', 'name': 'Valid Pairs FF' }
        keys['Valid_interaction_pairs_RR'] = { 'color': '#6699ff', 'name': 'Valid Pairs RR' }
        keys['Valid_interaction_pairs_RF'] = { 'color': '#0055ff', 'name': 'Valid Pairs RF' }
        keys['Valid_interaction_pairs_FR'] = { 'color': '#003399', 'name': 'Valid Pairs FR' }
        keys['Self_Cycle_pairs'] = { 'color': '#ffad99', 'name': 'Same Fragment - Self-Circle' }
        keys['Dangling_end_pairs'] = { 'color': '#ff5c33', 'name': 'Same Fragment - Dangling Ends' }
        keys['Religation_pairs'] = { 'color': '#cc2900', 'name': 'Re-ligation' }
        keys['Filtered_pairs'] = { 'color': '#661400', 'name': 'Filtered pairs' }
        keys['Dumped_pairs'] = { 'color': '#330a00', 'name': 'Dumped pairs' }

        # Config for the plot
        config = {
            'id': 'hicpro_filtering_plot',
            'title': 'HiC-Pro: Filtering Statistics',
            'ylab': '# Read Pairs',
            'cpswitch_counts_label': 'Number of Read Pairs'
        }
        
        return bargraph.plot(self.hicpro_data, keys, config)

    def hicpro_contact_chart (self):
        """ Generate the HiC-Pro interaction plot """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['cis_shortRange'] = { 'color': '#0039e6', 'name': 'Unique: cis <= 20Kbp' }
        keys['cis_longRange'] = { 'color': '#809fff', 'name': 'Unique: cis > 20Kbp' }
        keys['trans_interaction']  = { 'color': '#009933', 'name': 'Unique: trans' }
        keys['duplicates'] = { 'color': '#a9a2a2', 'name': 'Duplicate read pairs' }
        
        # Config for the plot
        config = {
            'id': 'hicpro_contact_plot',
            'title': 'HiC-Pro: Contact Statistics',
            'ylab': '# Pairs',
            'cpswitch_counts_label': 'Number of Pairs'
        }
        
        return bargraph.plot(self.hicpro_data, keys, config)
  
    def hicpro_as_chart (self):
        """ Generate Allele-specific plot"""
        
        keys = OrderedDict()
        keys['Valid_pairs_from_ref_genome_(1-1)'] = { 'color': '#e6550d', 'name': 'Genome1 specific read pairs (1-1)' }
        keys['Valid_pairs_from_ref_genome_with_one_unassigned_mate_(0-1/1-0)'] = { 'color': '#fdae6b', 'name': 'Genome1 with one unassigned mate (0-1/1-0)' }
        keys['Valid_pairs_from_alt_genome_(2-2)']  = { 'color': '#756bb1', 'name': 'Genome2 specific read pairs (2-2)' }
        keys['Valid_pairs_from_alt_genome_with_one_unassigned_mate_(0-2/2-0)'] = { 'color': '#bcbddc', 'name': 'Genome2 with one unassigned mate (0-2/2-0)' }
        keys['Valid_pairs_from_alt_and_ref_genome_(1-2/2-1)'] = { 'color': '#a6611a', 'name': 'Trans homologuous read pairs (1-2/2/1)' }
        keys['Valid_pairs_with_both_unassigned_mated_(0-0)'] = { 'color': '#cccccc', 'name': 'Unassigned read pairs' }
        keys['Valid_pairs_with_at_least_one_conflicting_mate_(3-)'] = { 'color': '#a9a2a2', 'name': 'Conflicting read pairs' }

        # check allele-specific analysis was run
        num_samples = 0
        for s_name in self.hicpro_data:
            for k in keys:
                num_samples += sum([1 if k in self.hicpro_data[s_name] else 0])
        if num_samples == 0:
            return False
        
        # Config for the plot
        config = {
            'id': 'hicpro_asan_plot',
            'title': 'HiC-Pro: Allele-specific Statistics',
            'ylab': '# Pairs',
            'cpswitch_counts_label': 'Number of Pairs'
        }

        return bargraph.plot(self.hicpro_data, keys, config)


    def hicpro_capture_chart (self):
        """ Generate Capture Hi-C plot"""

        keys = OrderedDict()
        keys['valid_pairs_on_target_cap_cap'] = { 'color': '#0039e6', 'name': 'Capture-Capture interactions' }
        keys['valid_pairs_on_target_cap_rep']  = { 'color': '#809fff', 'name': 'Capture-Reporter interactions' }
        keys['valid_pairs_off_target'] = { 'color': '#cccccc', 'name': 'Off-target valid pairs' }
  
        # Check capture info are available
        num_samples = 0
        for s_name in self.hicpro_data:
            for k in keys:
                num_samples += sum([1 if k in self.hicpro_data[s_name] else 0])
        if num_samples == 0:
            return False

        # Config for the plot
        config = {
            'id': 'hicpro_cap_plot',
            'title': 'HiC-Pro: Capture Statistics',
            'ylab': '# Pairs',
            'cpswitch_counts_label': 'Number of Pairs'
        }

        return bargraph.plot(self.hicpro_data, keys, config)


        



#!/usr/bin/env python

""" MultiQC module to parse output from STAR """

from __future__ import print_function
from collections import OrderedDict
import logging
import os
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='STAR', anchor='star',
        href="https://github.com/alexdobin/STAR",
        info="is an ultrafast universal RNA-seq aligner.")

        # Find and load any STAR reports
        self.star_data = dict()
        for f in self.find_log_files('star'):
            parsed_data = self.parse_star_report(f['f'])
            if parsed_data is not None:
                s_name = f['s_name']
                if s_name == '' or s_name == 'Log.final.out':
                    s_name = self.clean_s_name(os.path.basename(f['root']), os.path.dirname(f['root']))
                if s_name in self.star_data:
                    log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, section='SummaryLog')
                self.star_data[s_name] = parsed_data

        # Find and load any STAR gene count tables
        self.star_genecounts_unstranded = dict()
        self.star_genecounts_first_strand = dict()
        self.star_genecounts_second_strand = dict()
        for f in self.find_log_files('star/genecounts', filehandles=True):
            parsed_data = self.parse_star_genecount_report(f)
            if parsed_data is not None:
                s_name = f['s_name']
                if s_name == '' or s_name == 'ReadsPerGene.out.tab':
                    s_name = self.clean_s_name(os.path.basename(f['root']), os.path.dirname(f['root']))
                if s_name in self.star_data:
                    log.debug("Duplicate ReadsPerGene sample name found! Overwriting: {}".format(s_name))
                self.add_data_source(f, section='ReadsPerGene')
                self.star_genecounts_unstranded[s_name] = parsed_data['unstranded']
                self.star_genecounts_first_strand[s_name] = parsed_data['first_strand']
                self.star_genecounts_second_strand[s_name] = parsed_data['second_strand']

        # Filter to strip out ignored sample names
        self.star_data = self.ignore_samples(self.star_data)
        self.star_genecounts_unstranded = self.ignore_samples(self.star_genecounts_unstranded)
        self.star_genecounts_first_strand = self.ignore_samples(self.star_genecounts_first_strand)
        self.star_genecounts_second_strand = self.ignore_samples(self.star_genecounts_second_strand)

        if len(self.star_data) == 0 and len(self.star_genecounts_unstranded) == 0:
            raise UserWarning

        if len(self.star_data) > 0:
            if len(self.star_genecounts_unstranded) > 0:
                log.info("Found {} reports and {} gene count files".format(len(self.star_data), len(self.star_genecounts_unstranded)))
            else:
                log.info("Found {} reports".format(len(self.star_data)))
        else:
            log.info("Found {} gene count files".format(len(self.star_genecounts_unstranded)))

        if len(self.star_data) > 0:

            # Write parsed report data to a file
            self.write_data_file(self.star_data, 'multiqc_star')

            # Basic Stats Table
            self.star_stats_table()

            # Alignment bar plot
            self.add_section (
                name = 'Alignment Scores',
                anchor = 'star_alignments',
                plot = self.star_alignment_chart()
            )

        if len(self.star_genecounts_unstranded) > 0:
            self.add_section (
                name = 'Gene Counts',
                anchor = 'star_geneCounts',
                description = "Statistics from results generated using <code>--quantMode GeneCounts</code>. " +
                           "The three tabs show counts for unstranded RNA-seq, counts for the 1st read strand " +
                           "aligned with RNA and counts for the 2nd read strand aligned with RNA.",
                plot = self.star_genecount_chart()
            )


    def parse_star_report (self, raw_data):
        """ Parse the final STAR log file. """

        regexes = {
            'total_reads':                  r"Number of input reads \|\s+(\d+)",
            'avg_input_read_length':        r"Average input read length \|\s+([\d\.]+)",
            'uniquely_mapped':              r"Uniquely mapped reads number \|\s+(\d+)",
            'uniquely_mapped_percent':      r"Uniquely mapped reads % \|\s+([\d\.]+)",
            'avg_mapped_read_length':       r"Average mapped length \|\s+([\d\.]+)",
            'num_splices':                  r"Number of splices: Total \|\s+(\d+)",
            'num_annotated_splices':        r"Number of splices: Annotated \(sjdb\) \|\s+(\d+)",
            'num_GTAG_splices':             r"Number of splices: GT/AG \|\s+(\d+)",
            'num_GCAG_splices':             r"Number of splices: GC/AG \|\s+(\d+)",
            'num_ATAC_splices':             r"Number of splices: AT/AC \|\s+(\d+)",
            'num_noncanonical_splices':     r"Number of splices: Non-canonical \|\s+(\d+)",
            'mismatch_rate':                r"Mismatch rate per base, % \|\s+([\d\.]+)",
            'deletion_rate':                r"Deletion rate per base \|\s+([\d\.]+)",
            'deletion_length':              r"Deletion average length \|\s+([\d\.]+)",
            'insertion_rate':               r"Insertion rate per base \|\s+([\d\.]+)",
            'insertion_length':             r"Insertion average length \|\s+([\d\.]+)",
            'multimapped':                  r"Number of reads mapped to multiple loci \|\s+(\d+)",
            'multimapped_percent':          r"% of reads mapped to multiple loci \|\s+([\d\.]+)",
            'multimapped_toomany':          r"Number of reads mapped to too many loci \|\s+(\d+)",
            'multimapped_toomany_percent':  r"% of reads mapped to too many loci \|\s+([\d\.]+)",
            'unmapped_mismatches_percent':  r"% of reads unmapped: too many mismatches \|\s+([\d\.]+)",
            'unmapped_tooshort_percent':    r"% of reads unmapped: too short \|\s+([\d\.]+)",
            'unmapped_other_percent':       r"% of reads unmapped: other \|\s+([\d\.]+)",
        }
        parsed_data = {}
        for k, r in regexes.items():
            r_search = re.search(r, raw_data, re.MULTILINE)
            if r_search:
                parsed_data[k] = float(r_search.group(1))
        # Figure out the numbers for unmapped as for some reason only the percentages are given
        try:
            total_mapped = parsed_data['uniquely_mapped'] + parsed_data['multimapped'] + parsed_data['multimapped_toomany']
            unmapped_count = parsed_data['total_reads'] - total_mapped
            total_unmapped_percent = parsed_data['unmapped_mismatches_percent'] + parsed_data['unmapped_tooshort_percent'] + parsed_data['unmapped_other_percent']
            try:
                parsed_data['unmapped_mismatches'] = int(round(unmapped_count * (parsed_data['unmapped_mismatches_percent'] / total_unmapped_percent), 0))
                parsed_data['unmapped_tooshort'] = int(round(unmapped_count * (parsed_data['unmapped_tooshort_percent'] / total_unmapped_percent), 0))
                parsed_data['unmapped_other'] = int(round(unmapped_count * (parsed_data['unmapped_other_percent'] / total_unmapped_percent), 0))
            except ZeroDivisionError:
                parsed_data['unmapped_mismatches'] = 0
                parsed_data['unmapped_tooshort'] = 0
                parsed_data['unmapped_other'] = 0
        except KeyError:
            pass

        if len(parsed_data) == 0: return None
        return parsed_data

    def parse_star_genecount_report(self, f):
        """ Parse a STAR gene counts output file """
        # Three numeric columns: unstranded, stranded/first-strand, stranded/second-strand
        keys = [ 'N_unmapped', 'N_multimapping', 'N_noFeature', 'N_ambiguous' ]
        unstranded = { 'N_genes': 0 }
        first_strand = { 'N_genes': 0 }
        second_strand = { 'N_genes': 0 }
        num_errors = 0
        num_genes = 0
        for l in f['f']:
            s = l.split("\t")
            try:
                for i in [1,2,3]:
                    s[i] = float(s[i])
                if s[0] in keys:
                    unstranded[s[0]] = s[1]
                    first_strand[s[0]] = s[2]
                    second_strand[s[0]] = s[3]
                else:
                    unstranded['N_genes'] += s[1]
                    first_strand['N_genes'] += s[2]
                    second_strand['N_genes'] += s[3]
                    num_genes += 1
            except IndexError:
                # Tolerate a few errors in case there is something random added at the top of the file
                num_errors += 1
                if num_errors > 10 and num_genes == 0:
                    log.warning("Error parsing {}".format(f['fn']))
                    return None
        if num_genes > 0:
            return { 'unstranded': unstranded, 'first_strand': first_strand, 'second_strand': second_strand }
        else:
            return None

    def star_stats_table(self):
        """ Take the parsed stats from the STAR report and add them to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['uniquely_mapped_percent'] = {
            'title': '% Aligned',
            'description': '% Uniquely mapped reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['uniquely_mapped'] = {
            'title': '{} Aligned'.format(config.read_count_prefix),
            'description': 'Uniquely mapped reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.star_data, headers)

    def star_alignment_chart (self):
        """ Make the plot showing alignment rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['uniquely_mapped'] =      { 'color': '#437bb1', 'name': 'Uniquely mapped' }
        keys['multimapped'] =          { 'color': '#7cb5ec', 'name': 'Mapped to multiple loci' }
        keys['multimapped_toomany'] =  { 'color': '#f7a35c', 'name': 'Mapped to too many loci' }
        keys['unmapped_mismatches'] =  { 'color': '#e63491', 'name': 'Unmapped: too many mismatches' }
        keys['unmapped_tooshort'] =    { 'color': '#b1084c', 'name': 'Unmapped: too short' }
        keys['unmapped_other'] =       { 'color': '#7f0000', 'name': 'Unmapped: other' }

        # Config for the plot
        pconfig = {
            'id': 'star_alignment_plot',
            'title': 'STAR: Alignment Scores',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.star_data, keys, pconfig)

    def star_genecount_chart (self):
        """ Make a plot for the ReadsPerGene output """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['N_genes'] =        { 'color': '#2f7ed8', 'name': 'Overlapping Genes' }
        keys['N_noFeature'] =    { 'color': '#0d233a', 'name': 'No Feature' }
        keys['N_ambiguous'] =    { 'color': '#492970', 'name': 'Ambiguous Features' }
        keys['N_multimapping'] = { 'color': '#f28f43', 'name': 'Multimapping' }
        keys['N_unmapped'] =     { 'color': '#7f0000', 'name': 'Unmapped' }

        # Config for the plot
        pconfig = {
            'id': 'star_gene_counts',
            'title': 'STAR: Gene Counts',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'data_labels': ['Unstranded','Same Stranded','Reverse Stranded']
        }
        datasets = [
            self.star_genecounts_unstranded,
            self.star_genecounts_first_strand,
            self.star_genecounts_second_strand
        ]
        return bargraph.plot(datasets, [keys,keys,keys,keys], pconfig)
